@@ -2,23 +2,11 @@
 # Associate internals original text with physical files.
 
 
-$key = q/SubSec:Merge/;
+$key = q/Sec:git/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:propedit/;
-$ref_files{$key} = "$dir".q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/Sec:SVN/;
-$ref_files{$key} = "$dir".q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:conf/;
-$ref_files{$key} = "$dir".q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Bugs/;
+$key = q/SubSec:Contrib/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
@@ -26,10 +14,38 @@ $key = q/SubSec:Conflicts/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/SubSec:BeDev/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:conf/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/Sec:SVN/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Merge/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Bugs/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:propedit/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
 $key = q/SubSec:Inst/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/SubSec:qeforge/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
 $key = q/SubSec:CPP/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
