@@ -209,9 +209,6 @@
      LOGICAL :: dofft( nfftx ), done
      INTEGER, PARAMETER  :: stdout = 6
 
-#if defined __HPM
-     INTEGER :: OMP_GET_THREAD_NUM
-#endif
 #if defined(__OPENMP)
      INTEGER :: offset
      INTEGER :: nx_t, ny_t, nzl_t, ldx_t, ldy_t
