@@ -6,6 +6,307 @@
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
+!=----------------------------------------------------------------------=
+   MODULE block_distro
+!=----------------------------------------------------------------------=
+
+!  ... Added by Eric Pascolo
+
+
+   IMPLICIT NONE
+   PRIVATE
+   SAVE
+
+   PUBLIC :: map_blocks, find_max,write_matrix_strange_idx
+
+!=----------------------------------------------------------------------=
+   CONTAINS
+!=----------------------------------------------------------------------=
+
+
+SUBROUTINE map_blocks(maps,row_w,col_w,ub,lb,myid,num_of_core)
+
+  IMPLICIT NONE
+  INTEGER, INTENT(in) :: ub(3),lb(3),row_w,col_w,myid,num_of_core
+  INTEGER, INTENT(inout) :: maps( lb(1): ub(1), lb(2):ub(2) )
+  INTEGER :: i,nprow,npcol
+  INTEGER :: row_el,col_el,block_num
+  INTEGER, ALLOCATABLE :: proc_idx_matrix(:,:)
+  
+! GENERAZIONE DIMENSIONI BLOCCHI
+  CALL set_block(nprow,npcol,num_of_core)
+  
+! GENERAZIONE INFO BLOCCHI
+  CALL get_info_block(row_w,col_w,nprow,npcol,row_el,col_el,block_num)
+
+ 
+!  WRITE(6,*) '  PARAMETRI DISTRIBUZIONE G scalapack like'
+!  WRITE(6,*) '  -----------------------------------------'
+!  WRITE(6,*) '  Number of CORE', num_of_core
+!  WRITE(6,*) '  Number of ROW', ub(1)-lb(1)
+!  WRITE(6,*) '  Number of ROW wave', row_w
+!  WRITE(6,*) '  Number of COL', ub(2)-lb(2)
+!  WRITE(6,*) '  Number of COL wave', col_w
+!  WRITE(6,*) '  Number of core/ROW', nprow
+!  WRITE(6,*) '  Number of core/COL', npcol
+!  WRITE(6,*) '  SIZE ROW block', row_el
+!  WRITE(6,*) '  SIZE COL block', col_el
+!  WRITE(6,*) '  NUMBER OF BLOCK', block_num
+!  WRITE(6,*) '  -----------------------------------------'
+
+  ALLOCATE(proc_idx_matrix(nprow,npcol))
+  
+  proc_idx_matrix = 0
+  maps = 0
+        
+! GENERAZIONE MATRICE PER DISTRIBUZIONE PROCESSORI 
+  CALL set_matrix_processor(nprow,npcol,proc_idx_matrix)
+  
+! SCRITTURA SU FILE MATRICE PER DISTRIBUZIONE PROCESSORI
+!  IF(myid .eq. 0) CALL write_matrix(proc_idx_matrix,nprow,npcol,"./mappe/pattern_proc.matrix")
+  
+! GENERAZIONE MATRICE MAPPA 
+  CALL distro_matrix_processor(ub,lb,nprow,npcol,row_el,col_el,proc_idx_matrix,maps)
+  
+!  IF(myid .eq. 0) CALL write_matrix_strange_idx(maps,ub,lb,"./mappe/mappa_core_scalike.matrix")
+    
+  DEALLOCATE(proc_idx_matrix)
+    
+END SUBROUTINE map_blocks
+
+!=----------------------------------------------------------------------=
+
+SUBROUTINE set_block(npr,npc,num_of_core)
+! GENERAZIONE DIMENSIONI BLOCCHI
+  IMPLICIT NONE
+  INTEGER, INTENT(in) :: num_of_core
+  INTEGER, INTENT(inout) :: npr,npc
+  INTEGER :: i,sqrtnp
+
+  sqrtnp = INT( SQRT( REAL( num_of_core ) + 0.1 ) )
+  
+  DO i = 1, sqrtnp + 1
+    IF( MOD( num_of_core, i ) == 0 ) npr = i
+  ENDDO
+
+  npc = num_of_core / npr
+
+END SUBROUTINE set_block
+
+!=----------------------------------------------------------------------=
+
+SUBROUTINE get_info_block(row,col,npr,npc,elpr,elpc,epb)
+! CALCOLO INFO BLOCCHI
+  IMPLICIT NONE
+  INTEGER, INTENT(in) :: row,col
+  INTEGER, INTENT(in) :: npr,npc
+  INTEGER, INTENT(out) :: elpr,elpc,epb
+  INTEGER :: i,sqrtnp
+  
+  elpr = (row/npr);
+  elpc = (col/npc);
+  epb =  ((row*col)/(2*elpr*elpc));
+  
+END SUBROUTINE get_info_block
+
+!=----------------------------------------------------------------------=
+
+SUBROUTINE set_matrix_processor(npr,npc,pmatrix)
+! GENERAZIONE MATRICE PROCESSORI
+  IMPLICIT NONE
+  INTEGER, INTENT(in) :: npr,npc
+  INTEGER, INTENT(out) :: pmatrix(:,:)
+  INTEGER :: i,j,p,row,col
+  
+  p = 1
+  DO i=1,npc
+    DO j=1,npr
+        pmatrix(j,i) = p
+        p= p+1
+    ENDDO
+  ENDDO    
+  
+  
+END SUBROUTINE set_matrix_processor
+
+!=----------------------------------------------------------------------=
+
+SUBROUTINE distro_matrix_processor(ub,lb,npr,npc,elpr,elpc,pmatrix,proc_distro_matrix)
+
+! DISTRO
+
+  IMPLICIT NONE
+  INTEGER, INTENT(in) :: npr,npc,elpr,elpc,ub(3),lb(3)
+  INTEGER, INTENT(in) :: pmatrix(npr,npc)
+  INTEGER, INTENT(inout) :: proc_distro_matrix(lb(1): ub(1), lb(2):ub(2))
+  INTEGER :: i,j,i0,j0,i0b,j0b,ib,jb,jb1,row,col
+  proc_distro_matrix = 0
+  ib = 0
+  jb = 0
+  row = ub(1)
+  col = ub(2)
+  
+!   RIEMPIMENTO CENTRO RIGHT
+  DO i=elpr,row,elpr
+    
+    ib = MOD(ib,npr)+1;
+    i0=i - elpr;
+    jb = 0
+    DO j=elpc,col,elpc
+           
+      j0=j - elpc;
+      jb = MOD(jb,npc)+1;
+      
+      proc_distro_matrix(i0:i,j0:j) = pmatrix(ib,jb); 
+      j0b = j
+    ENDDO
+    i0b = i
+  ENDDO
+
+     
+  ib = 0
+  
+!   RIEMPIMENTO BORDO RIGHT LATERALE
+  DO i=0,row
+  
+      proc_distro_matrix(i,j0b:col) = proc_distro_matrix(i,j0b-1) 
+        
+  ENDDO
+    
+!   RIEMPIMENTO BORDO RIGHT SOPRA
+  DO j=0,col
+      proc_distro_matrix(i0b:row,j) =  proc_distro_matrix(i0b-1,j);
+  ENDDO
+  
+  !RIEMPIMENTO CENTRO LEFT
+  ib = 0
+  row = ub(1)
+  col = lb(2)
+
+  DO i=elpr,row,elpr
+    
+    ib = MOD(ib,npr)+1;
+    jb1 = 0
+    DO j=-elpc-1,col,-elpc
+       
+      i0=i - elpr;
+      j0=j + elpc;
+      jb = npc - MOD(jb1,npc);
+      proc_distro_matrix(i0:i,j:j0) = pmatrix(ib,jb);
+      jb1 = jb1+1
+      j0b = j
+
+    ENDDO
+    
+    i0b = i
+    
+  ENDDO
+  
+  ib = 2
+  
+  !RIEMPIMENTO BORDO LEFT LATERALE
+  DO i=0,row
+      proc_distro_matrix(i,col:j0b) = proc_distro_matrix(i,j0b+1) 
+  ENDDO
+    
+  !RIEMPIMENTO BORDO LEFT SOTTO
+  DO j=-1,col,-1
+      proc_distro_matrix(i0b:ub(1),j) =  proc_distro_matrix(i0b-1,j);
+  ENDDO
+  
+  
+END SUBROUTINE distro_matrix_processor
+
+!=----------------------------------------------------------------------=
+
+SUBROUTINE find_max(ub,lb,mtw,row,col,my,ncore)
+
+  INTEGER,INTENT(OUT) :: row,col
+  INTEGER,INTENT(IN)  :: ub(:),lb(:),mtw(lb(1):ub(1),lb(2):ub(2))
+  INTEGER,INTENT(IN)  :: my,ncore
+  INTEGER :: i,j,max_proc,vector(0:ub(2)),ncr,ncc
+  
+  CALL set_block(ncr,ncc,ncore)
+  vector = 0
+  max_proc = max(ncr,ncc)
+  
+  WRITE(6,*) 'max proc',max_proc,'ncr',ncr,'ncc',ncc
+  WRITE(6,*) 'ub1',ub(1),'ub2',ub(2)
+  
+  DO j=0,ub(2)
+  
+    DO i=0,ub(1)
+    
+    IF (mtw(i,j).ne. 0)THEN
+      vector(j) = i
+    ELSE
+      EXIT
+    ENDIF
+    
+    ENDDO
+        
+  ENDDO
+  
+  
+  IF(my .eq. 1) WRITE(*,*) 'vector',vector(:)
+  
+  
+  DO i=0,ub(2)
+  
+     IF( abs(i-vector(i)) .le. 1 .AND. i>max_proc .AND. vector(i)>max_proc ) THEN
+     row = vector(i)
+     col = i
+     EXIT
+     
+     ENDIF
+  
+  ENDDO
+  
+ 
+  
+END SUBROUTINE
+
+!=----------------------------------------------------------------------=
+
+SUBROUTINE write_matrix(mtw,righe,colonne,percorso)
+
+    IMPLICIT NONE
+    INTEGER :: i, j, righe,colonne
+    character(LEN=*), INTENT(in) :: percorso
+    INTEGER, INTENT(in) :: mtw(righe,colonne)
+    
+    OPEN(unit=115, file=percorso)
+    DO i=1,righe
+       WRITE(115,'(1000I6)') mtw(i,1:colonne)
+    END DO
+    CLOSE(115)
+
+END SUBROUTINE write_matrix
+
+!=----------------------------------------------------------------------=
+
+SUBROUTINE write_matrix_strange_idx(mtw,ub,lb,percorso)
+
+  IMPLICIT NONE
+  INTEGER :: i, j, ub(3),lb(3)
+  character(LEN=*), INTENT(in) :: percorso
+  INTEGER, INTENT(in) :: mtw(lb(1):ub(1),lb(2):ub(2))
+    
+  OPEN(unit=116, file=percorso)
+
+    DO i=ub(1),lb(1),-1
+       WRITE(116,'(1000I6)') mtw(i,lb(2):ub(2))
+    END DO
+    
+  CLOSE(116)
+
+END SUBROUTINE write_matrix_strange_idx
+
+!=----------------------------------------------------------------------=
+   END MODULE block_distro
+!=----------------------------------------------------------------------=
+
+
 !=----------------------------------------------------------------------=
    MODULE stick_base
 !=----------------------------------------------------------------------=
@@ -81,12 +382,12 @@
                     gsq=    (dble(i)*b1(1)+dble(j)*b2(1)+dble(k)*b3(1) )**2
                     gsq=gsq+(dble(i)*b1(2)+dble(j)*b2(2)+dble(k)*b3(2) )**2
                     gsq=gsq+(dble(i)*b1(3)+dble(j)*b2(3)+dble(k)*b3(3) )**2
-                    IF(gsq.le.gcut ) THEN
+                    IF(gsq<=gcut ) THEN
                       st(i,j) = st(i,j) + 1
-                      IF(gsq.le.gcutw) THEN
+                      IF(gsq<=gcutw) THEN
                         stw(i,j) = stw(i,j) + 1
                       ENDIF
-                      IF(gsq.le.gcuts) THEN
+                      IF(gsq<=gcuts) THEN
                         sts(i,j) = sts(i,j) + 1
                       ENDIF
                     ENDIF
@@ -103,12 +404,12 @@
                     gsq=    (dble(i)*b1(1)+dble(j)*b2(1)+dble(k)*b3(1) )**2
                     gsq=gsq+(dble(i)*b1(2)+dble(j)*b2(2)+dble(k)*b3(2) )**2
                     gsq=gsq+(dble(i)*b1(3)+dble(j)*b2(3)+dble(k)*b3(3) )**2
-                    IF(gsq.le.gcut ) THEN
+                    IF(gsq<=gcut ) THEN
                       st(i,j) = st(i,j) + 1
-                      IF(gsq.le.gcutw) THEN
+                      IF(gsq<=gcutw) THEN
                         stw(i,j) = stw(i,j) + 1
                       ENDIF
-                      IF(gsq.le.gcuts) THEN
+                      IF(gsq<=gcuts) THEN
                         sts(i,j) = sts(i,j) + 1
                       ENDIF
                     ENDIF
@@ -125,12 +426,12 @@
                     gsq=    (dble(i)*b1(1)+dble(j)*b2(1)+dble(k)*b3(1) )**2
                     gsq=gsq+(dble(i)*b1(2)+dble(j)*b2(2)+dble(k)*b3(2) )**2
                     gsq=gsq+(dble(i)*b1(3)+dble(j)*b2(3)+dble(k)*b3(3) )**2
-                    IF(gsq.le.gcut ) THEN
+                    IF(gsq<=gcut ) THEN
                       st(i,j) = st(i,j) + 1
-                      IF(gsq.le.gcutw) THEN
+                      IF(gsq<=gcutw) THEN
                         stw(i,j) = stw(i,j) + 1
                       ENDIF
-                      IF(gsq.le.gcuts) THEN
+                      IF(gsq<=gcuts) THEN
                         sts(i,j) = sts(i,j) + 1
                       ENDIF
                     ENDIF
@@ -149,13 +450,13 @@
                     gsq=    (dble(i)*b1(1)+dble(j)*b2(1)+dble(k)*b3(1) )**2
                     gsq=gsq+(dble(i)*b1(2)+dble(j)*b2(2)+dble(k)*b3(2) )**2
                     gsq=gsq+(dble(i)*b1(3)+dble(j)*b2(3)+dble(k)*b3(3) )**2
-                    IF(gsq.le.gcut ) THEN
+                    IF(gsq<=gcut ) THEN
                       st(i,j) = st(i,j) + 1
                     ENDIF
-                    IF(gsq.le.gcutw) THEN
+                    IF(gsq<=gcutw) THEN
                       stw(i,j) = stw(i,j) + 1
                     ENDIF
-                    IF(gsq.le.gcuts) THEN
+                    IF(gsq<=gcuts) THEN
                       sts(i,j) = sts(i,j) + 1
                     ENDIF
                   ENDIF
@@ -166,9 +467,9 @@
           ENDIF
 
 #if defined(__MPI)
-          CALL MPI_ALLREDUCE(MPI_IN_PLACE, st, SIZE(st), MPI_INTEGER, MPI_SUM, comm, ierr)
-          CALL MPI_ALLREDUCE(MPI_IN_PLACE, stw, SIZE(stw), MPI_INTEGER, MPI_SUM, comm, ierr)
-          CALL MPI_ALLREDUCE(MPI_IN_PLACE, sts, SIZE(sts), MPI_INTEGER, MPI_SUM, comm, ierr)
+          CALL MPI_ALLREDUCE(MPI_IN_PLACE, st, size(st), MPI_INTEGER, MPI_SUM, comm, ierr)
+          CALL MPI_ALLREDUCE(MPI_IN_PLACE, stw, size(stw), MPI_INTEGER, MPI_SUM, comm, ierr)
+          CALL MPI_ALLREDUCE(MPI_IN_PLACE, sts, size(sts), MPI_INTEGER, MPI_SUM, comm, ierr)
 #endif
 
         RETURN
@@ -275,9 +576,9 @@
             aux(mc) = dn3 * aux(mc) + ngcs(mc)
             aux(mc) = dn3 * aux(mc) + ngc(mc)
             aux(mc) = -aux(mc)
-            idx(mc) = 0 
+            idx(mc) = 0
           ENDDO
-          CALL hpsort( nct, aux(1), idx(1))
+          CALL hpsort( nct, aux, idx)
           DEALLOCATE( aux )
         ELSE
           ic = 0
@@ -364,8 +665,8 @@
             in2(nct) = i2
 
             ngc(nct) = st( i1 , i2)
-            IF( stw( i1, i2 ) .gt. 0 ) ngcw(nct) = stw( i1 , i2)
-            IF( sts( i1, i2 ) .gt. 0 ) ngcs(nct) = sts( i1 , i2)
+            IF( stw( i1, i2 ) > 0 ) ngcw(nct) = stw( i1 , i2)
+            IF( sts( i1, i2 ) > 0 ) ngcs(nct) = sts( i1 , i2)
 
           ENDIF
 
@@ -380,13 +681,20 @@
 !=----------------------------------------------------------------------=
 
     SUBROUTINE sticks_dist( tk, ub, lb, idx, in1, in2, ngc, ngcw, ngcs, nct, &
-                            ncp, ncpw, ncps, ngp, ngpw, ngps, stown, stownw, stowns, nproc )
+                            ncp, ncpw, ncps, ngp, ngpw, ngps, stown, stownw, stowns, mype, nproc )
+#ifdef __NEW_DISTRO
+      USE block_distro
+#endif
 
       LOGICAL, INTENT(in) :: tk
 
       INTEGER, INTENT(in) :: ub(:), lb(:), idx(:)
       INTEGER, INTENT(out) :: stown( lb(1): ub(1), lb(2):ub(2) ) ! stick map for potential
+#ifdef __NEW_DISTRO
+      INTEGER, INTENT(inout) :: stownw(lb(1): ub(1), lb(2):ub(2) ) ! stick map for wave functions
+#else
       INTEGER, INTENT(out) :: stownw(lb(1): ub(1), lb(2):ub(2) ) ! stick map for wave functions
+#endif
       INTEGER, INTENT(out) :: stowns(lb(1): ub(1), lb(2):ub(2) ) ! stick map for smooth mesh
 
       INTEGER, INTENT(in) :: in1(:), in2(:)
@@ -394,9 +702,12 @@
       INTEGER, INTENT(in) :: nct
       INTEGER, INTENT(out) :: ncp(:), ncpw(:), ncps(:)
       INTEGER, INTENT(out) :: ngp(:), ngpw(:), ngps(:)
+      INTEGER, INTENT(in) :: mype  ! my proc id in the g-vec group
       INTEGER, INTENT(in) :: nproc ! number of proc in the g-vec group
 
-      INTEGER :: mc, i1, i2, i, j, jj, icnt
+      INTEGER, ALLOCATABLE :: maps(:,:) ! maps
+
+      INTEGER :: mc, i1, i2, i, j, jj, icnt, rw, cl
 
       ncp  = 0
       ncps = 0
@@ -406,14 +717,63 @@
       ngpw = 0
 
       stown  = 0
+#if ! defined __NEW_DISTRO
       stownw = 0
+#endif
       stowns = 0
 
       icnt = 0
 
+#ifdef __NEW_DISTRO
+
+      ALLOCATE(maps(lb(1): ub(1), lb(2):ub(2) ))
+
+      CALL find_max(ub,lb,stownw,rw,cl,mype+1,nproc)
+
+      CALL map_blocks(maps,rw,cl,ub,lb,mype+1,nproc)
+
       DO mc = 1, nct
 
          i = idx( mc )
+
+         i1 = in1( i )
+         i2 = in2( i )
+
+         IF ( ( .not. tk ) .and. ( (i1 < 0) .or. ( (i1 == 0) .and. (i2 < 0) ) )) GOTO 31
+
+         jj = maps(i1,i2)
+
+         ncp(jj) = ncp(jj) + 1
+         ngp(jj) = ngp(jj) + ngc(i)
+         stown(i1,i2) = jj
+
+         ! smooth mesh
+
+         IF ( ngcs(i) > 0 ) THEN
+            ncps(jj) = ncps(jj) + 1
+            ngps(jj) = ngps(jj) + ngcs(i)
+            stowns(i1,i2) = jj
+         ENDIF
+
+         ! wave functions mesh
+
+         IF ( ngcw(i) > 0 ) THEN
+            ncpw(jj) = ncpw(jj) + 1
+            ngpw(jj) = ngpw(jj) + ngcw(i)
+            stownw(i1,i2) = jj
+         ENDIF
+
+ 31      CONTINUE
+
+      ENDDO
+
+      DEALLOCATE(maps)
+
+#else
+
+
+      DO mc = 1, nct
+
          i = idx( mc )
 !
 ! index contains the desired ordering of sticks (see above)
@@ -478,6 +838,8 @@
 
       ENDDO
 
+#endif
+
       RETURN
     END SUBROUTINE sticks_dist
 
@@ -558,7 +920,7 @@
                             ncp, ncpw, ncps, ngp, ngpw, ngps, stown, stownw, stowns, nproc )
 !
 ! This routine works as sticks_dist only it distributes the sticks according to sticks_owner.
-! This ensures that the gvectors for any 'smooth like grid' remain on the same proc as the 
+! This ensures that the gvectors for any 'smooth like grid' remain on the same proc as the
 ! original grid.
 !
       LOGICAL, INTENT(in) :: tk
@@ -601,7 +963,7 @@
 !
          ! potential mesh set according to sticks_owner
 
-         jj = stown(i1,i2) 
+         jj = stown(i1,i2)
          ncp(jj) = ncp(jj) + 1
          ngp(jj) = ngp(jj) + ngc(i)
 
@@ -629,8 +991,114 @@
     END SUBROUTINE sticks_ordered_dist
 
 !=----------------------------------------------------------------------=
-
-
+    
+!---------------------------------------------------------------------
+    SUBROUTINE hpsort (n, ra, ind)
+      !---------------------------------------------------------------------
+      ! sort an array ra(1:n) into ascending order using heapsort algorithm.
+      ! n is input, ra is replaced on output by its sorted rearrangement.
+      ! create an index table (ind) by making an exchange in the index array
+      ! whenever an exchange is made on the sorted data array (ra).
+      ! in case of equal values in the data array (ra) the values in the
+      ! index array (ind) are used to order the entries.
+      ! if on input ind(1)  = 0 then indices are initialized in the routine,
+      ! if on input ind(1) != 0 then indices are assumed to have been
+      !                initialized before entering the routine and these
+      !                indices are carried around during the sorting process
+      !
+      ! no work space needed !
+      ! free us from machine-dependent sorting-routines !
+      !
+      ! adapted from Numerical Recipes pg. 329 (new edition)
+      !
+      IMPLICIT NONE
+      !-input/output variables
+      INTEGER :: n
+      INTEGER :: ind (n)
+      REAL(DP) :: ra (n)
+      !-local variables
+      INTEGER :: i, ir, j, l, iind
+      REAL(DP) :: rra
+      ! initialize index array
+      IF (ind (1) ==0) THEN
+         DO i = 1, n
+            ind (i) = i
+         ENDDO
+      ENDIF
+      ! nothing to order
+      IF (n<2) RETURN
+      ! initialize indices for hiring and retirement-promotion phase
+      l = n / 2 + 1
+      ir = n
+10    CONTINUE
+      ! still in hiring phase
+      IF (l>1) THEN
+         l = l - 1
+         rra = ra (l)
+         iind = ind (l)
+         ! in retirement-promotion phase.
+      ELSE
+         ! clear a space at the end of the array
+         rra = ra (ir)
+         !
+         iind = ind (ir)
+         ! retire the top of the heap into it
+         ra (ir) = ra (1)
+         !
+         ind (ir) = ind (1)
+         ! decrease the size of the corporation
+         ir = ir - 1
+         ! done with the last promotion
+         IF (ir==1) THEN
+            ! the least competent worker at all !
+            ra (1) = rra
+            !
+            ind (1) = iind
+            RETURN
+         ENDIF
+      ENDIF
+      ! wheter in hiring or promotion phase, we
+      i = l
+      ! set up to place rra in its proper level
+      j = l + l
+      !
+      DO WHILE (j<=ir)
+         IF (j<ir) THEN
+            ! compare to better underling
+            IF (ra (j) <ra (j + 1) ) THEN
+               j = j + 1
+            ELSEIF (ra (j) ==ra (j + 1) ) THEN
+               IF (ind (j) <ind (j + 1) ) j = j + 1
+            ENDIF
+         ENDIF
+         ! demote rra
+         IF (rra<ra (j) ) THEN
+            ra (i) = ra (j)
+            ind (i) = ind (j)
+            i = j
+            j = j + j
+         ELSEIF (rra==ra (j) ) THEN
+            ! demote rra
+            IF (iind<ind (j) ) THEN
+               ra (i) = ra (j)
+               ind (i) = ind (j)
+               i = j
+               j = j + j
+            ELSE
+               ! set j to terminate do-while loop
+               j = ir + 1
+            ENDIF
+            ! this is the right place for rra
+         ELSE
+            ! set j to terminate do-while loop
+            j = ir + 1
+         ENDIF
+      ENDDO
+      ra (i) = rra
+      ind (i) = iind
+      GOTO 10
+      !
+    END SUBROUTINE hpsort
 
     SUBROUTINE sticks_deallocate
       IF( allocated( sticks_owner ) ) DEALLOCATE( sticks_owner )
