@@ -38,7 +38,7 @@ MODULE mytime
   !
   SAVE
   !
-  INTEGER,  PARAMETER :: maxclock = 100
+  INTEGER,  PARAMETER :: maxclock = 101
   REAL(DP), PARAMETER :: notrunning = - 1.0_DP
   !
   REAL(DP)          :: cputime(maxclock), t0cpu(maxclock)
@@ -48,7 +48,9 @@ MODULE mytime
   !
   INTEGER :: nclock = 0
   LOGICAL :: no
+#if defined (__TRACE)
   INTEGER :: trace_depth = 0
+#endif
   !
 END MODULE mytime
 !
@@ -61,15 +63,15 @@ SUBROUTINE init_clocks( go )
   !
   USE kinds,  ONLY : DP
   USE mytime, ONLY : called, t0cpu, cputime, no, notrunning, maxclock, &
-       clock_label, walltime, t0wall
+       clock_label, walltime, t0wall, nclock
   !
   IMPLICIT NONE
   !
   LOGICAL :: go
   INTEGER :: n
   !
-  !
   no = .not. go
+  nclock = 0
   !
   DO n = 1, maxclock
      !
@@ -93,10 +95,11 @@ SUBROUTINE start_clock( label )
   USE kinds,     ONLY : DP
   USE io_global, ONLY : stdout
 #if defined (__TRACE)
-  USE mp_global, ONLY : mpime
+  USE mp_world,  ONLY : mpime
+  USE mytime,    ONLY : trace_depth
 #endif
   USE mytime,    ONLY : nclock, clock_label, notrunning, no, maxclock, &
-                        t0cpu, t0wall, trace_depth
+                        t0cpu, t0wall
   !
   IMPLICIT NONE
   !
@@ -142,7 +145,7 @@ SUBROUTINE start_clock( label )
   !
   IF ( nclock == maxclock ) THEN
      !
-     WRITE( stdout, '("start_clock: Too many clocks! call ignored")' )
+     WRITE( stdout, '("start_clock(",A,"): Too many clocks! call ignored")' ) label
      !
   ELSE
      !
@@ -164,10 +167,11 @@ SUBROUTINE stop_clock( label )
   USE kinds,     ONLY : DP
   USE io_global, ONLY : stdout
 #if defined (__TRACE)
-  USE mp_global, ONLY : mpime
+  USE mp_world,  ONLY : mpime
+  USE mytime,    ONLY : trace_depth
 #endif
   USE mytime,    ONLY : no, nclock, clock_label, cputime, walltime, &
-                        notrunning, called, t0cpu, t0wall, trace_depth
+                        notrunning, called, t0cpu, t0wall
   !
   IMPLICIT NONE
   !
@@ -228,7 +232,6 @@ END SUBROUTINE stop_clock
 SUBROUTINE print_clock( label )
   !----------------------------------------------------------------------------
   !
-  USE kinds,     ONLY : DP
   USE io_global, ONLY : stdout
   USE mytime,    ONLY : nclock, clock_label
   !
@@ -279,13 +282,13 @@ SUBROUTINE print_this_clock( n )
   !
   USE kinds,     ONLY : DP
   USE io_global, ONLY : stdout
-  USE mytime,    ONLY : no, nclock, clock_label, cputime, walltime, &
+  USE mytime,    ONLY : clock_label, cputime, walltime, &
                         notrunning, called, t0cpu, t0wall
 !
 ! ... See comments below about parallel case
 !
 !  USE mp,        ONLY : mp_max
-!  USE mp_global, ONLY : intra_image_comm, my_image_id
+!  USE mp_images, ONLY : intra_image_comm, my_image_id
   !
   IMPLICIT NONE
   !
@@ -333,6 +336,14 @@ SUBROUTINE print_this_clock( n )
      !
      ! ... The first clock is written as days/hour/min/sec
      !
+#if defined(__CLOCK_SECONDS)
+     !
+     WRITE( stdout, &
+        '(5X,A12," : ",F9.2,"s CPU ",F9.2,"s WALL"/)' ) &
+        clock_label(n), elapsed_cpu_time, elapsed_wall_time
+     !
+#else
+     !
      nday  = elapsed_cpu_time / 86400
      nsec  = elapsed_cpu_time - 86400 * nday
      nhour = nsec / 3600
@@ -377,6 +388,7 @@ SUBROUTINE print_this_clock( n )
              clock_label(n), nsec, msec
         !
      ENDIF
+#endif
      !
   ELSEIF ( nmax == 1 .or. t0cpu(n) /= notrunning ) THEN
      !
@@ -413,14 +425,13 @@ FUNCTION get_clock( label )
   !----------------------------------------------------------------------------
   !
   USE kinds,     ONLY : DP
-  USE io_global, ONLY : stdout
   USE mytime,    ONLY : no, nclock, clock_label, walltime, &
-                        notrunning, called, t0wall, t0cpu
+                        notrunning, t0wall, t0cpu
 !
 ! ... See comments in subroutine print_this_clock about parallel case
 !
 !  USE mp,        ONLY : mp_max
-!  USE mp_global, ONLY : intra_image_comm
+!  USE mp_images, ONLY : intra_image_comm
   !
   IMPLICIT NONE
   !
