@@ -444,6 +444,7 @@ MODULE input_parameters
           ! Various parameters for noncollinear calculations
         LOGICAL  :: noncolin = .false.
         LOGICAL  :: lspinorb = .false.
+        LOGICAL  :: lforcet=.FALSE.
         LOGICAL  :: starting_spin_angle=.FALSE.
         REAL(DP) :: lambda = 1.0_DP
         REAL(DP) :: fixed_magnetization(3) = 0.0_DP
@@ -488,9 +489,11 @@ MODULE input_parameters
           ! other DFT-D parameters ( see Modules/mm_dispersion.f90 )
           ! london_s6 = default global scaling parameter for PBE
           ! london_c6 = user specified atomic C6 coefficients
+          ! london_rvdw = user specified atomic vdw radii
         REAL ( DP ) :: london_s6   =   0.75_DP , &
                        london_rcut = 200.00_DP , &
-                       london_c6( nsx ) = -1.0_DP
+                       london_c6( nsx ) = -1.0_DP, &
+                       london_rvdw( nsx ) = -1.0_DP
 
         LOGICAL   :: ts_vdw = .false.
           ! OBSOLESCENT: same as vdw_corr='Tkatchenko-Scheffler'
@@ -581,11 +584,11 @@ MODULE input_parameters
              exxdiv_treatment, x_gamma_extrapolation, yukawa, ecutvcut,       &
              exx_fraction, screening_parameter, ref_alat,                     &
              noncolin, lspinorb, starting_spin_angle, lambda, angle1, angle2, &
-             report,              &
+             report, lforcet,                                                 &
              constrained_magnetization, B_field, fixed_magnetization,         &
              sic, sic_epsilon, force_pairing, sic_alpha,                      &
              tot_charge, tot_magnetization, spline_ps, one_atom_occupations,  &
-             vdw_corr, london, london_s6, london_rcut, london_c6,             &
+             vdw_corr, london, london_s6, london_rcut, london_c6, london_rvdw,&
              ts_vdw, ts_vdw_isolated, ts_vdw_econv_thr,                       &
              xdm, xdm_a1, xdm_a2,                                             &
              step_pen, A_pen, sigma_pen, alpha_pen, no_t_rev,                 &
