@@ -11,6 +11,8 @@
 MODULE xml_input
 
    USE xml_io_base, ONLY : attr
+   USE qexml_module, ONLY: qexml_init, qexml_write_header, qexml_openfile, &
+                           qexml_closefile
    USE iotk_module
    USE kinds
 
@@ -35,8 +37,6 @@ MODULE xml_input
       CHARACTER(LEN=256) :: filename
       INTEGER            :: ierr
 
-      return
-
       filename = 'qe_input.xml'
       
       IF ( ionode ) THEN
@@ -45,7 +45,8 @@ MODULE xml_input
          !
          WRITE( stdout, '(/,3X,"Dumping input parameters",/)' )
          !
-         CALL iotk_open_write( iunpun, FILE = filename, BINARY = .FALSE., IERR = ierr )
+         CALL qexml_init( iunpun )
+         CALL qexml_openfile( filename, 'write', .FALSE., ierr )
          !
       END IF
 
@@ -57,7 +58,7 @@ MODULE xml_input
          CALL iotk_write_attr( attr, "xmlns:tns", "http://www.deisa.org/pwscf/3_2" )
          CALL iotk_write_begin( iunpun, "schema", attr )
 
-         CALL write_header( "Quantum ESPRESSO", TRIM(version_number) )
+         CALL qexml_write_header( "Quantum ESPRESSO", TRIM(version_number) )
 
          CALL iotk_write_attr( attr, "section_type", "namelist", FIRST = .TRUE. )
          CALL iotk_write_begin( iunpun, "CONTROLS", attr )
@@ -101,7 +102,7 @@ MODULE xml_input
 
       END IF
 
-      IF ( ionode ) CALL iotk_close_write( iunpun )
+      IF ( ionode ) CALL qexml_closefile( 'write', IERR=ierr )
 
       RETURN
    END SUBROUTINE
