@@ -220,7 +220,7 @@ PROGRAM PAWplot
   !   set default values for variables in namelist
   !
   prefix = 'pwscf'
-  CALL get_env( 'ESPRESSO_TMPDIR', outdir )
+  CALL get_environment_variable( 'ESPRESSO_TMPDIR', outdir )
   IF ( trim( outdir ) == ' ' ) outdir = './'
   filplot = 'pawcharge.dat'
   plot    = 'valence'
