@@ -85,11 +85,11 @@ SUBROUTINE new_hamiltonian(form, plot_bands)
   USE io_files
   USE kinds, ONLY: DP
   USE wannier_new, ONLY: nwan, pp, wannier_occ, wannier_energy, wan_in
-  USE klist, ONLY: nks, xk, wk
+  USE klist, ONLY: nks, xk, wk, igk_k
   USE lsda_mod, ONLY: isk, current_spin, lsda, nspin
-  USE wvfct, ONLY: nbnd, npwx, igk, npw, g2kin, et,  ecutwfc
+  USE wvfct, ONLY: nbnd, npwx, igk, npw, g2kin, et
+  USE gvecw, ONLY : gcutw
   USE gvect
-  USE cell_base, ONLY: tpiba2
   USE constants,  ONLY : rytoev , tpi
   USE buffers
   USE symm_base,  ONLY : nsym
@@ -128,10 +128,9 @@ SUBROUTINE new_hamiltonian(form, plot_bands)
   CALL init_at_1
 
   ! Generating igk for orthoatwfc()
-  REWIND( iunigk )
+
   DO ik = 1, nks
-     CALL gk_sort( xk(1,ik), ngm, g, ecutwfc / tpiba2, npw, igk, g2kin )
-     IF ( nks > 1 ) WRITE( iunigk ) igk
+     CALL gk_sort( xk(1,ik), ngm, g, gcutw, npw, igk_k(1,ik), g2kin )
   ENDDO
   !
   CALL orthoatwfc( .true. )
@@ -142,7 +141,7 @@ SUBROUTINE new_hamiltonian(form, plot_bands)
 
   DO ik = 1, nks
      write(stdout,*) '       Computing k-point', ik
-     CALL gk_sort (xk (1, ik), ngm, g, ecutwfc / tpiba2, npw, igk, g2kin)
+     CALL gk_sort (xk (1, ik), ngm, g, gcutw, npw, igk, g2kin)
      IF (lsda) current_spin  = isk(ik)
      CALL wannier_proj(ik,wan_func)
 
@@ -367,4 +366,4 @@ SUBROUTINE plot_wannier_bands(ek)
   CLOSE(114)
   CLOSE(115)
 
-END SUBROUTINE plot_wannier_bands
\ No newline at end of file
+END SUBROUTINE plot_wannier_bands
