@@ -2,14 +2,14 @@
 # Associate labels original text with physical files.
 
 
-$key = q/pw2casino_info/;
-$external_labels{$key} = "$URL/" . q|node12.html|; 
-$noresave{$key} = "$nosave";
-
 $key = q/SubSec:badpara/;
 $external_labels{$key} = "$URL/" . q|node17.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/pw2casino_info/;
+$external_labels{$key} = "$URL/" . q|node12.html|; 
+$noresave{$key} = "$nosave";
+
 1;
 
 
@@ -17,13 +17,17 @@ $noresave{$key} = "$nosave";
 # labels from external_latex_labels array.
 
 
-$key = q/SubSec:badpara/;
-$external_latex_labels{$key} = q|4.4|; 
-$noresave{$key} = "$nosave";
-
 $key = q/pw2casino_info/;
 $external_latex_labels{$key} = q|3.5|; 
 $noresave{$key} = "$nosave";
 
+$key = q/_oldnewlabel/;
+$external_latex_labels{$key} = q|<|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:badpara/;
+$external_latex_labels{$key} = q|4.4|; 
+$noresave{$key} = "$nosave";
+
 1;
 
