@@ -157,15 +157,15 @@ SUBROUTINE c_phase
    USE io_files,             ONLY : iunwfc, nwordwfc
    USE buffers,              ONLY : get_buffer
    USE ions_base,            ONLY : nat, ntyp => nsp, ityp, tau, zv, atm
-   USE cell_base,            ONLY : at, alat, tpiba, omega, tpiba2
+   USE cell_base,            ONLY : at, alat, tpiba, omega
    USE constants,            ONLY : pi, tpi
    USE gvect,                ONLY : ngm, g, gcutm, ngm_g, ig_l2g
    USE fft_base,             ONLY : dfftp
    USE uspp,                 ONLY : nkb, vkb, okvan
    USE uspp_param,           ONLY : upf, lmaxq, nbetam, nh, nhm
    USE lsda_mod,             ONLY : nspin
-   USE klist,                ONLY : nelec, degauss, nks, xk, wk
-   USE wvfct,                ONLY : npwx, npw, nbnd, ecutwfc, wg
+   USE klist,                ONLY : nelec, degauss, nks, xk, wk, igk_k, ngk
+   USE wvfct,                ONLY : npwx, nbnd, wg
    USE wavefunctions_module, ONLY : evc
    USE bp,                   ONLY : gdir, nppstr, mapgm_global, pdl_tot
    USE becmod,               ONLY : calbec, bec_type, allocate_bec_type, &
@@ -238,7 +238,6 @@ SUBROUTINE c_phase
    REAL(DP) :: el_loc
    REAL(DP) :: eps
    REAL(DP) :: fac
-   REAL(DP) :: g2kin_bp(npwx)
    REAL(DP) :: gpar(3)
    REAL(DP) :: gtr(3)
    REAL(DP) :: gvec
@@ -474,8 +473,8 @@ SUBROUTINE c_phase
             IF (kpar /= 1) THEN
 
 !              --- Dot wavefunctions and betas for PREVIOUS k-point ---
-               CALL gk_sort(xk(1,kpoint-1),ngm,g,ecutwfc/tpiba2, &
-                            npw0,igk0,g2kin_bp) 
+               npw0 = ngk(kpoint-1)
+               igk0(:) = igk_k(:,kpoint-1)
                CALL get_buffer (psi,nwordwfc,iunwfc,kpoint-1)
                if (okvan) then
                   CALL init_us_2 (npw0,igk0,xk(1,kpoint-1),vkb)
@@ -483,8 +482,8 @@ SUBROUTINE c_phase
                endif
 !              --- Dot wavefunctions and betas for CURRENT k-point ---
                IF (kpar /= nppstr) THEN
-                  CALL gk_sort(xk(1,kpoint),ngm,g,ecutwfc/tpiba2, &
-                               npw1,igk1,g2kin_bp)        
+                  npw1 = ngk(kpoint)
+                  igk1(:) = igk_k(:,kpoint)
                   CALL get_buffer(evc,nwordwfc,iunwfc,kpoint)
                   if (okvan) then
                      CALL init_us_2 (npw1,igk1,xk(1,kpoint),vkb)
@@ -492,8 +491,8 @@ SUBROUTINE c_phase
                   endif
                ELSE
                   kstart = kpoint-nppstr+1
-                  CALL gk_sort(xk(1,kstart),ngm,g,ecutwfc/tpiba2, &
-                               npw1,igk1,g2kin_bp)  
+                  npw1 = ngk(kstart)
+                  igk1(:) = igk_k(:,kstart)
                   CALL get_buffer(evc,nwordwfc,iunwfc,kstart)
                   if (okvan) then
                      CALL init_us_2 (npw1,igk1,xk(1,kstart),vkb)
