@@ -28,15 +28,15 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
    USE io_files,             ONLY : iunwfc, nwordwfc,prefix,tmp_dir
    USE buffers,              ONLY : get_buffer
    USE ions_base,            ONLY : nat, ntyp => nsp, ityp, tau, zv, atm
-   USE cell_base,            ONLY : at, alat, tpiba, omega, tpiba2
+   USE cell_base,            ONLY : at, alat, tpiba, omega
    USE constants,            ONLY : pi, tpi
    USE fft_base,             ONLY : dfftp
    USE gvect,                ONLY : ngm, g, gcutm, ngm_g
    USE uspp,                 ONLY : nkb, vkb, okvan
    USE uspp_param,           ONLY : upf, lmaxq, nbetam, nh, nhm
    USE lsda_mod,             ONLY : nspin
-   USE klist,                ONLY : nelec, degauss, nks, xk, wk
-   USE wvfct,                ONLY : npwx, npw, nbnd, ecutwfc
+   USE klist,                ONLY : nelec, degauss, nks, xk, wk, ngk, igk_k
+   USE wvfct,                ONLY : npwx, nbnd
    USE noncollin_module,     ONLY : noncolin, npol
    USE bp,                   ONLY : nppstr_3d, mapgm_global, nx_el,phase_control
    USE fixed_occ
@@ -55,7 +55,7 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
    INTEGER, INTENT(in) :: pdir!direction on which the polarization is calculated
 
 !  --- Internal definitions ---
-   INTEGER :: i
+   INTEGER :: i, ik
    INTEGER :: igk1(npwx)
    INTEGER :: igk0(npwx)
    INTEGER :: ig
@@ -101,7 +101,6 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
    REAL(dp) :: el_loc
    REAL(dp) :: eps
    REAL(dp) :: fac
-   REAL(dp) :: g2kin_bp(npwx)
    REAL(dp) :: gpar(3)
    REAL(dp) :: gtr(3)
    REAL(dp) :: gvec
@@ -397,8 +396,9 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
             IF (kpar /= 1 ) THEN
              
 !              --- Dot wavefunctions and betas for PREVIOUS k-point ---
-               CALL gk_sort(xk(1,nx_el(kpoint-1,pdir)),ngm,g,ecutwfc/tpiba2, &
-                            npw0,igk0,g2kin_bp) 
+               ik = nx_el(kpoint-1,pdir)
+               npw0   = ngk(ik)
+               igk0(:)= igk_k(:,ik)
                CALL get_buffer (psi,nwordwfc,iunwfc,nx_el(kpoint-1,pdir))
                if (okvan) then
                   CALL init_us_2 (npw0,igk0,xk(1,nx_el(kpoint-1,pdir)),vkb)
@@ -406,8 +406,9 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
                endif
 !              --- Dot wavefunctions and betas for CURRENT k-point ---
                IF (kpar /= (nppstr_3d(pdir)+1)) THEN
-                  CALL gk_sort(xk(1,nx_el(kpoint,pdir)),ngm,g,ecutwfc/tpiba2, &
-                               npw1,igk1,g2kin_bp)        
+                  ik = nx_el(kpoint,pdir)
+                  npw1   = ngk(ik)
+                  igk1(:)= igk_k(:,ik)
                   CALL get_buffer (psi1,nwordwfc,iunwfc,nx_el(kpoint,pdir))
                   if(okvan) then
                      CALL init_us_2 (npw1,igk1,xk(1,nx_el(kpoint,pdir)),vkb)
@@ -415,8 +416,9 @@ SUBROUTINE c_phase_field(el_pola,ion_pola, fact_pola, pdir)
                   endif
                ELSE
                   kstart = kpoint-(nppstr_3d(pdir)+1)+1
-                  CALL gk_sort(xk(1,nx_el(kstart,pdir)),ngm,g,ecutwfc/tpiba2, &
-                               npw1,igk1,g2kin_bp)  
+                  ik = nx_el(kstart,pdir)
+                  npw1   = ngk(ik)
+                  igk1(:)= igk_k(:,ik)
                   CALL get_buffer (psi1,nwordwfc,iunwfc,nx_el(kstart,pdir))
                   if(okvan) then
                      CALL init_us_2 (npw1,igk1,xk(1,nx_el(kstart,pdir)),vkb)
