@@ -18,12 +18,12 @@ SUBROUTINE orthoUwfc
   USE kinds,      ONLY : DP
   USE buffers,    ONLY : get_buffer, save_buffer
   USE io_global,  ONLY : stdout
-  USE io_files,   ONLY : iunhub, nwordwfcU, iunigk
+  USE io_files,   ONLY : iunhub, nwordwfcU
   USE ions_base,  ONLY : nat
   USE basis,      ONLY : natomwfc, swfcatom
-  USE klist,      ONLY : nks, xk, ngk
+  USE klist,      ONLY : nks, xk, ngk, igk_k
   USE ldaU,       ONLY : U_projection, wfcU, nwfcU, copy_U_wfc
-  USE wvfct,      ONLY : npwx, npw, igk
+  USE wvfct,      ONLY : npwx
   USE uspp,       ONLY : nkb, vkb
   USE becmod,     ONLY : allocate_bec_type, deallocate_bec_type, &
                          bec_type, becp, calbec
@@ -34,7 +34,7 @@ SUBROUTINE orthoUwfc
   !
   !
   INTEGER :: ik, ibnd, info, i, j, k, na, nb, nt, isym, n, ntemp, m, &
-       l, lm, ltot, ntot, ipol
+       l, lm, ltot, ntot, ipol, npw
   ! ik: the k point under consideration
   ! ibnd: counter on bands
   LOGICAL :: orthogonalize_wfc, normalize_only
@@ -79,19 +79,15 @@ SUBROUTINE orthoUwfc
   ! Allocate the array becp = <beta|wfcatom>
   CALL allocate_bec_type (nkb,natomwfc, becp) 
   
-  IF (nks > 1) REWIND (iunigk)
-  
   DO ik = 1, nks
      
-     npw = ngk (ik)
-     IF (nks > 1) READ (iunigk) igk
-     
      IF (noncolin) THEN
        CALL atomic_wfc_nc_updown (ik, wfcatom)
      ELSE
        CALL atomic_wfc (ik, wfcatom)
      ENDIF
-     CALL init_us_2 (npw, igk, xk (1, ik), vkb)
+     npw = ngk (ik)
+     CALL init_us_2 (npw, igk_k(1,ik), xk (1, ik), vkb)
      CALL calbec (npw, vkb, wfcatom, becp) 
      CALL s_psi (npwx, npw, natomwfc, wfcatom, swfcatom)
 
@@ -125,11 +121,11 @@ SUBROUTINE orthoatwfc (orthogonalize_wfc)
   USE kinds,      ONLY : DP
   USE buffers,    ONLY : save_buffer
   USE io_global,  ONLY : stdout
-  USE io_files,   ONLY : iunsat, nwordatwfc, iunigk
+  USE io_files,   ONLY : iunsat, nwordatwfc
   USE ions_base,  ONLY : nat
   USE basis,      ONLY : natomwfc, swfcatom
-  USE klist,      ONLY : nks, xk, ngk
-  USE wvfct,      ONLY : npwx, npw, igk
+  USE klist,      ONLY : nks, xk, ngk, igk_k
+  USE wvfct,      ONLY : npwx
   USE uspp,       ONLY : nkb, vkb
   USE becmod,     ONLY : allocate_bec_type, deallocate_bec_type, &
                          bec_type, becp, calbec
@@ -141,7 +137,7 @@ SUBROUTINE orthoatwfc (orthogonalize_wfc)
   LOGICAL, INTENT(in) :: orthogonalize_wfc
   !
   INTEGER :: ik, ibnd, info, i, j, k, na, nb, nt, isym, n, ntemp, m, &
-       l, lm, ltot, ntot, ipol
+       l, lm, ltot, ntot, ipol, npw
   ! ik: the k point under consideration
   ! ibnd: counter on bands
   LOGICAL :: normalize_only = .FALSE.
@@ -153,19 +149,15 @@ SUBROUTINE orthoatwfc (orthogonalize_wfc)
   ! Allocate the array becp = <beta|wfcatom>
   CALL allocate_bec_type (nkb,natomwfc, becp) 
   
-  IF (nks > 1) REWIND (iunigk)
-  
   DO ik = 1, nks
      
-     npw = ngk (ik)
-     IF (nks > 1) READ (iunigk) igk
-     
      IF (noncolin) THEN
        CALL atomic_wfc_nc_updown (ik, wfcatom)
      ELSE
        CALL atomic_wfc (ik, wfcatom)
      ENDIF
-     CALL init_us_2 (npw, igk, xk (1, ik), vkb)
+     npw = ngk (ik)
+     CALL init_us_2 (npw, igk_k(1,ik), xk (1, ik), vkb)
      CALL calbec (npw, vkb, wfcatom, becp) 
      CALL s_psi (npwx, npw, natomwfc, wfcatom, swfcatom)
 
