@@ -1352,7 +1352,7 @@ SUBROUTINE PAW_dpotential(dbecsum, becsum, int3, npe)
                                                            ! occupations 
    COMPLEX(DP), INTENT(IN) :: dbecsum(nhm*(nhm+1)/2,nat,nspin_mag,npe)! 
    
-   COMPLEX(DP), INTENT(OUT) :: int3(nhm,nhm,npe,nat,nspin_mag) ! change of 
+   COMPLEX(DP), INTENT(OUT) :: int3(nhm,nhm,nat,nspin_mag,npe) ! change of 
                                            !descreening coefficients (AE - PS)
    INTEGER, PARAMETER      :: AE = 1, PS = 2,&      ! All-Electron and Pseudo
                               XC = 1, H  = 2        ! XC and Hartree
@@ -1507,12 +1507,12 @@ SUBROUTINE PAW_dpotential(dbecsum, becsum, int3, npe)
                                             savedvi_lm(1:i%m,lm,is,ipert) 
                            CALL simpson (upf(i%t)%kkbeta,aux_lm, &
                                                       g(i%t)%rab,integral_i)
-                           int3(nb,mb,ipert,i%a,is) = &
-                                        int3(nb,mb,ipert,i%a,is) &
+                           int3(nb,mb,i%a,is,ipert) = &
+                                        int3(nb,mb,i%a,is,ipert) &
                                        + sgn * CMPLX(integral_r, integral_i,kind=DP)
                         ENDDO
-                        IF (nb /= mb)  int3(mb,nb,ipert,i%a,is) = &
-                                                    int3(nb,mb,ipert,i%a,is) 
+                        IF (nb /= mb)  int3(mb,nb,i%a,is,ipert) = &
+                                                    int3(nb,mb,i%a,is,ipert) 
                      ENDDO
                      becfake(nmb,ia,is) = 0._dp
                   ENDDO ! mb
