@@ -627,11 +627,11 @@ SUBROUTINE extrapolate_wfcs( wfc_extr )
   ! ... by Mead, Rev. Mod. Phys., vol 64, pag. 51 (1992), eqs. 3.20-3.29
   !
   USE io_global,            ONLY : stdout
-  USE klist,                ONLY : nks, ngk, xk
+  USE klist,                ONLY : nks, ngk, xk, igk_k
   USE lsda_mod,             ONLY : lsda, current_spin, isk
-  USE wvfct,                ONLY : nbnd, npw, npwx, igk, current_k
+  USE wvfct,                ONLY : nbnd, npwx, current_k
   USE ions_base,            ONLY : nat, tau
-  USE io_files,             ONLY : nwordwfc, iunigk, iunwfc, iunoldwfc, &
+  USE io_files,             ONLY : nwordwfc, iunwfc, iunoldwfc, &
                                    iunoldwfc2, diropn
   USE buffers,              ONLY : get_buffer, save_buffer
   USE uspp,                 ONLY : nkb, vkb, okvan
@@ -647,7 +647,7 @@ SUBROUTINE extrapolate_wfcs( wfc_extr )
   !
   INTEGER, INTENT(IN) :: wfc_extr
   !
-  INTEGER :: ik, zero_ew, lwork, info
+  INTEGER :: npw, ik, zero_ew, lwork, info
     ! do-loop variables
     ! counter on k-points
     ! number of zero 'eigenvalues' of the s_m matrix
@@ -723,8 +723,6 @@ SUBROUTINE extrapolate_wfcs( wfc_extr )
         ALLOCATE( work( lwork ) )
      END IF
      !
-     IF ( nks > 1 ) REWIND( iunigk )
-     !
      zero_ew = 0
      !
      DO ik = 1, nks
@@ -735,25 +733,18 @@ SUBROUTINE extrapolate_wfcs( wfc_extr )
         IF ( nks > 1 ) CALL get_buffer( evc, nwordwfc, iunwfc, ik )
         CALL davcio(    evc, 2*nwordwfc, iunoldwfc, ik, +1 )
         !
+        npw = ngk (ik)
         IF ( okvan ) THEN
            !
            ! ... Ultrasoft PP: calculate overlap matrix
-           ! ... various initializations: k, spin, number of PW, indices
-           !
-           current_k = ik
-           IF ( lsda ) current_spin = isk(ik)
-           npw = ngk (ik)
-           IF ( nks > 1 ) READ( iunigk ) igk
-           !
-           call g2_kin (ik)
-           !
-           ! ... Calculate nonlocal pseudopotential projectors |beta>
-           !
-           IF ( nkb > 0 ) CALL init_us_2( npw, igk, xk(1,ik), vkb )
+           ! ... Required by s_psi:
+           ! ... nonlocal pseudopotential projectors |beta>, <psi|beta>
            !
+           IF ( nkb > 0 ) CALL init_us_2( npw, igk_k(1,ik), xk(1,ik), vkb )
            CALL calbec( npw, vkb, evc, becp )
            !
            CALL s_psi ( npwx, npw, nbnd, evc, aux )
+           !
         ELSE
            !
            ! ... Norm-Conserving  PP: no overlap matrix
