@@ -1,36 +1,43 @@
-#!/bin/bash
+!/bin/bash
 
 #
 # CIF to Quantum Espresso format converter
+#  Version 1.2  Date: 20-Mar-2015  Bug Fix
+#  Version 1.1  Date: 23-Dec-2014  Bug Fix
+#               Date: 30-Set-2014  -s option added (F. Zadra)
 #  Version 1.0  Date: 15-Mar-2014  First Full conversion
 #  Version 0.5  Date: 02-Oct-2013
 #  Version 0.4  Date: 12 Jun 2013
 #  Version 0.3  Date: 15 Nov 2012
 #
-# Copyright (C) 2012-2014 Carlo Nervi
+# Copyright (C) 2012-2015 Carlo Nervi
 # This file is distributed under the terms of the
 # GNU General Public License. See the file `License'
 # in the root directory of the present distribution,
 # or http://www.gnu.org/copyleft/gpl.txt .
 #
-# Tested with GNU awk v.4 - doesn't work with earlier gawk versions
 # Use dos2unix to strip carriage returns at the end of the .cif files!!!!
+# Tested with GNU awk v.4 - doesn't work with earlier gawk versions
 #
 # symmetry x,-y+1/2,z+y without ''
 #
 
-version="1.0"
-USAGE="cif2qe.sh Version ${version}\nUsage: cif2qe.sh [-i] File\n    ( -i uses the ibrav of QE. Do not add .cif extension!) -  Requires File.cif\n"
+version="1.2"
+USAGE="cif2qe.sh Version ${version}\nUsage: cif2qe.sh [-i] [-s] File\n    ( -i uses the ibrav of QE. -s uses CRYSTAL sg atom position mode.\n Do not add .cif extension!) -  Requires File.cif\n"
 if [ $# == 0 -o $# -gt 2 ]; then
  printf "$USAGE"
  exit
 fi
 
 do_ibrav=0
+sg=0
 if [ $# == 2 ]; then
  if [ $1 == "-i" ]; then
   do_ibrav=1
   shift
+ elif [ $1 == "-s" ]; then
+  sg=2
+  shift
  else
   printf "$USAGE"
   exit
@@ -42,7 +49,7 @@ if [ ! -f $1.cif ]; then
  exit
 fi
 
-awk -v FILE="$1" -v VERSION="$version" -v do_IBRAV=$do_ibrav '
+awk -v FILE="$1" -v VERSION="$version" -v do_IBRAV=$do_ibrav -v SG=$sg '
 
 BEGIN {
  bohr = 0.52917721092
@@ -81,7 +88,7 @@ BEGIN {
  KeyW[0][11]="_symmetry_equiv_pos_as_xyz";   KeyW[1][11]="_space_group_symop_operation_xyz"
  KeyW[0][12]="_symmetry_cell_setting";       KeyW[1][12]="_space_group_crystal_system"
  KeyW[0][13]="_symmetry_Int_Tables_number";  KeyW[1][13]="_space_group_IT_number"
-
+ 
 #
 #Tolerance for recognize identical atoms generate by symmetry
 #
@@ -117,14 +124,15 @@ function parseX(str, field) {
 #
 # Consideriamo il testo fra due apostrofi singoli come una stringa
 # ritorna il campo field-esimo della stringa str
+# Se non ci sono i due apostrofi \047 ritorna il campo field-esimo
 #
- c=0
+ idx1=0         
  str=str " "
  while (str) {
-   match(str,/ *\47[^\47]*\47 * |[^ ]* /) 
+   match(str,/^ *[^ ]* |^ *\47[^\47]*\47 /)
    f=substr(str,RSTART,RLENGTH)             # save what matched in f
-   gsub(/^ *\47?|\47? * $/,"",f)               # remove extra stuff
-   if ( ++c == field) { if (f!="") return f
+   gsub(/^ *|^ *\47?|\47? *$/,"",f)               # remove extra stuff
+   if ( ++idx1 == field) { if (f!="") return f
      else return $field
    }
  str=substr(str,RLENGTH+1)                 # "consume" what matched
@@ -136,17 +144,14 @@ function search_K(str) {
  i0=0
  for (i1=1; i1<=nKey; i1++) {
    for (i2=0; i2<nsynon; i2++) {
-#     printf "KeyW[%i][%i]=%s == %s\n", i2, i1, KeyW[i2][i1], str
-#     if (KeyW[i2][i1] == str) printf "FOUND %s in [%i]\n", str, i1
      if (KeyW[i2][i1] == str) return i1
    }
  }
-# printf "[%s] NOT FOUND\n", str
  return 0
 }
 
 function eval(cmd,expression) {
- expression = "awk \047BEGIN{printf \"%19.15f\", " cmd " }\047"
+ expression = "awk \047BEGIN{printf \"%19.14f\", " cmd " }\047"
  expression |& getline l
  close(expression)
  return l     
@@ -288,8 +293,11 @@ END {
  if (tmptablenumber > 0 ) tmpspacegroup=Int_Tables[tmptablenumber]
  ibrav=Find_ibrav(tmpspacegroup, reticolo)
  KP_x = int(1./(a*separation)+0.5)
+ if (KP_x < 1) KP_x=1
  KP_y = int(1./(b*separation)+0.5)
+ if (KP_y < 1) KP_y=1
  KP_z = int(1./(c*separation)+0.5)
+ if (KP_z < 1) KP_z=1
 
  ntyp=0
  for (i=0; i<natom; i++) {
@@ -300,41 +308,29 @@ END {
   }
  }
 
- for (j=0; j<nsymm; j++) {
-  if (split(Array["_symmetry_equiv_pos_as_xyz"][j], Tmp, ",") != 3) {
-    print "Error in _symmetry_equiv_pos_as_xyz. Number of fields != 3: [1]=" Tmp[1] " [2]=" Tmp[2] " [3]=" Tmp[3]
-    print "D: " Array["_symmetry_equiv_pos_as_xyz"][j] "  " Tmp[1] "  " Tmp[2] "  " Tmp[3]
-    exit
-  }
-  TS[j][1]=Tmp[1]; TS[j][2]=Tmp[2]; TS[j][3]=Tmp[3]
-  x=strtonum(Array["_atom_site_fract_x"][0])
-  y=strtonum(Array["_atom_site_fract_y"][0])
-  z=strtonum(Array["_atom_site_fract_z"][0])
-  f_x=strtonum(Array["_atom_site_fract_x"][0])
-  f_y=strtonum(Array["_atom_site_fract_y"][0])
-  f_z=strtonum(Array["_atom_site_fract_z"][0])
-  p_X=norma(f_x, f_y, f_z, Tmp[1])
-  p_Y=norma(f_x, f_y, f_z, Tmp[2])
-  p_Z=norma(f_x, f_y, f_z, Tmp[3])
-  ff=0
-  for (jj=0; jj<j; jj++) {
-    for (i=0; i<natom; i++) {
-      f_x=strtonum(Array["_atom_site_fract_x"][i])
-      f_y=strtonum(Array["_atom_site_fract_y"][i])
-      f_z=strtonum(Array["_atom_site_fract_z"][i])
-      if (abs(p_X - norma(f_x, f_y, f_z, TS[jj][1])) < tol && abs(p_Y - norma(f_x, f_y, f_z, TS[jj][2])) < tol && abs(p_Z - norma(f_x, f_y, f_z, TS[jj][3])) < tol ) ff=1
+ for (i=0; i<natom; i++) {
+  f_x=strtonum(Array["_atom_site_fract_x"][i])
+  f_y=strtonum(Array["_atom_site_fract_y"][i])
+  f_z=strtonum(Array["_atom_site_fract_z"][i])
+  for (j=0; j<nsymm; j++) {
+   if (split(Array["_symmetry_equiv_pos_as_xyz"][j], Tmp, ",") != 3) {
+     print "Error in _symmetry_equiv_pos_as_xyz. Number of fields != 3: [1]=" Tmp[1] " [2]=" Tmp[2] " [3]=" Tmp[3]
+     print "D: " Array["_symmetry_equiv_pos_as_xyz"][j] "  " Tmp[1] "  " Tmp[2] "  " Tmp[3]
+     exit
+   }
+   p_X=norma(f_x, f_y, f_z, Tmp[1])
+   p_Y=norma(f_x, f_y, f_z, Tmp[2])
+   p_Z=norma(f_x, f_y, f_z, Tmp[3])   
+   ff=0
+   for (ii=0; ii<totatom; ii++) if (abs(p_X - save_X[ii]) < tol && abs(p_Y - save_Y[ii]) < tol && abs(p_Z - save_Z[ii]) < tol) ff=1
+    if (ff==0) {
+      save_S[totatom]=Array["_atom_site_type_symbol"][i]
+      save_X[totatom]=p_X
+      save_Y[totatom]=p_Y
+      save_Z[totatom]=p_Z
+      totatom++
     }
   }
-  if (ff==1) continue
-  for (i=0; i<natom; i++) {
-    f_x=strtonum(Array["_atom_site_fract_x"][i])
-    f_y=strtonum(Array["_atom_site_fract_y"][i])
-    f_z=strtonum(Array["_atom_site_fract_z"][i])
-    save_X[totatom]=norma(f_x, f_y, f_z, Tmp[1])
-    save_Y[totatom]=norma(f_x, f_y, f_z, Tmp[2])
-    save_Z[totatom]=norma(f_x, f_y, f_z, Tmp[3])
-  totatom++
-  }
  }
  print "! Generated by using cif2qe Version " VERSION " - Date: " strftime()
  print "!   _symmetry_space_group_name_H-M = "  Var2["_symmetry_space_group"]
@@ -343,6 +339,8 @@ END {
  print "! a=" a "  b=" b "  c=" c "  alpha=" alpha "  beta=" beta "  gamma=" gamma
  print "! Found by cif2qe: lattice = " reticolo "    Space group = " Int_Tables[tmptablenumber] "   ibrav = " ibrav
  print "!"
+ if (SG==2)  print "! Using CRYSTALL_sg input position mode"
+ print "!"
  print "! Symmetry found:"
  for (j=0; j<nsymm; j++) {
    printf "! %3i %30s    ", j+1, Array["_symmetry_equiv_pos_as_xyz"][j] 
@@ -365,42 +363,44 @@ END {
  print "                     tprnfor = .true."
  print " /"
  print " &SYSTEM"
+
+ if (SG!=2) {
  if (do_IBRAV == 1 && ibrav != 0) {
    print "                       ibrav = " ibrav
-   printf "                   celldm(1) = %19.15f\n", a/bohr
+   printf "                   celldm(1) = %19.14f\n", a/bohr
    switch (ibrav) {
     case 4:
     case 6:
     case 7:
-     printf "                   celldm(3) = %19.15f\n", c/a
+     printf "                   celldm(3) = %19.14f\n", c/a
      break
     case 5:
     case -5:
-     printf "                   celldm(4) = %19.15f\n", cos(alphar)
+     printf "                   celldm(4) = %19.14f\n", cos(alphar)
      break
     case 14:
-     printf "                   celldm(2) = %19.15f, celldm(3) = %19.15f\n", b/a, c/a
-     printf "                   celldm(4) = %19.15f\n", cos(alphar)
-     printf "                   celldm(5) = %19.15f\n", cos(betar)
-     printf "                   celldm(6) = %19.15f\n", cos(gammar)
+     printf "                   celldm(2) = %19.14f, celldm(3) = %19.14f\n", b/a, c/a
+     printf "                   celldm(4) = %19.14f\n", cos(alphar)
+     printf "                   celldm(5) = %19.14f\n", cos(betar)
+     printf "                   celldm(6) = %19.14f\n", cos(gammar)
      break
     case -12:
-     printf "                   celldm(2) = %19.15f, celldm(3) = %19.15f\n", b/a, c/a
-     printf "                   celldm(5) = %19.15f\n", cos(betar)
+     printf "                   celldm(2) = %19.14f, celldm(3) = %19.14f\n", b/a, c/a
+     printf "                   celldm(5) = %19.14f\n", cos(betar)
      break
     case 13:
-     printf "                   celldm(2) = %19.15f, celldm(3) = %19.15f\n", b/a, c/a
-     printf "                   celldm(4) = %19.15f\n", cos(gammar)
+     printf "                   celldm(2) = %19.14f, celldm(3) = %19.14f\n", b/a, c/a
+     printf "                   celldm(4) = %19.14f\n", cos(gammar)
      break
     case 8:
     case 9:
     case 10:
     case 11:
-     printf "                   celldm(2) = %19.15f, celldm(3) = %19.15f\n", b/a, c/a
+     printf "                   celldm(2) = %19.14f, celldm(3) = %19.14f\n", b/a, c/a
      break
     case 12:
-     printf "                   celldm(2) = %19.15f, celldm(3) = %19.15f\n", b/a, c/a
-     printf "                   celldm(4) = %19.15f\n", cos(gammar)
+     printf "                   celldm(2) = %19.14f, celldm(3) = %19.14f\n", b/a, c/a
+     printf "                   celldm(4) = %19.14f\n", cos(gammar)
      break
     case 1:
     case 2:
@@ -409,10 +409,22 @@ END {
  } else print "                       ibrav = 0"
  print "                         nat = " totatom
  print "                        ntyp = " ntyp
- print "                     ecutwfc = 50"
- print "                     ecutrho = 400"
- print "                      london = .true."
- print "                   london_s6 = 0.75"
+ }
+
+if (SG==2) {
+   print "                 space_group = " tmptablenumber
+   print "                rhombhoedral = .TRUE."
+   print "                    unique_b = .FALSE."
+   print "               origin_choice = 1"
+   print "                         nat = " natom
+   print "                        ntyp = " ntyp
+}
+
+ print "                     ecutwfc = 60"
+ print "                     ecutrho = 480"
+ print "                    vdw_corr = \x027" "xdm\x027"
+ print "                      xdm_a1 = 1.2153"
+ print "                      xdm_a2 = 2.3704"
  print " /"
  print " &ELECTRONS"
  print "            electron_maxstep = 200"
@@ -429,15 +441,24 @@ END {
  print "                ion_dynamics = \x027" "bfgs\x027"
  print " /"
  print "\nATOMIC_SPECIES"
- for (i=0; i<ntyp; i++) printf "  %3s  %14.10f  %s.pbe-n-rrkjus_psl.0.1.UPF\n", AtomTyp[i], Atoms[AtomTyp[i]], AtomTyp[i];
+ for (i=0; i<ntyp; i++) printf "  %3s  %14.10f  %s.pw86pbe-n-kjpaw_psl.1.0.0.UPF\n", AtomTyp[i], Atoms[AtomTyp[i]], AtomTyp[i];
 
+if (SG!=2) {
  print "\nATOMIC_POSITIONS crystal"
  for (i=0; i<totatom; i++)
-   printf "%2s   %19.15f   %19.15f   %19.15f\n", Array["_atom_site_type_symbol"][i % natom], save_X[i], save_Y[i], save_Z[i]
+   printf "%2s   %19.14f   %19.14f   %19.14f\n", save_S[i], save_X[i], save_Y[i], save_Z[i]
+}
+
+if (SG==2) {
+ print "\nATOMIC_POSITION crystal_sg"
+ for (i=0; i<natom; i++)
+   printf "%2s   %19.14f    %19.14f    %19.14f\n", save_S[i], save_X[i], save_Y[i], save_Z[i]
+}
+
  print "\nK_POINTS automatic"
  print KP_x "  " KP_y "  " KP_z "   0 0 0"
 
- if (do_IBRAV == 0) {
+ if (do_IBRAV == 0 || SG == 2) {
    cell_px[0]=a
    cell_py[0]=0.0
    cell_pz[0]=0.0
@@ -448,8 +469,9 @@ END {
    cell_py[2]=c*(cos(alphar)-cos(betar)*cos(gammar))/sin(gammar)
    cell_pz[2]=c*sqrt(1.0 - cos(alphar)^2 - cos(betar)^2 - cos(gammar)^2 + 2*cos(alphar)*cos(betar)*cos(gammar))/sin(gammar)
    print "\nCELL_PARAMETERS"
-   for (i=0; i<3; i++) printf "  %19.15f   %19.15f   %19.15f\n", cell_px[i]/bohr, cell_py[i]/bohr, cell_pz[i]/bohr;
+   for (i=0; i<3; i++) printf "  %19.14f   %19.14f   %19.14f\n", cell_px[i]/bohr, cell_py[i]/bohr, cell_pz[i]/bohr;
  }
  print "\n\n"
 
 } ' $1.cif
+
