@@ -32,6 +32,8 @@ rm archive/plumed-1.3-qe.tar.gz archive/PLUMED-latest.tar.gz
 
 # restore version.f90 
 mv version.f90 Modules/
+cp Modules/version.f90 Modules/version.f90.in
+chmod -x install/update_version
 
 # generate documentation - NOTA BENE:
 # in order to build the .html and .txt documentation in Doc, 
@@ -44,7 +46,6 @@ touch make.sys
 make doc
 
 # generate PWGUI
-
 make tar-gui PWGUI_VERSION=$version 
 tar -xzvf PWgui-$version.tgz
 /bin/rm PWgui-$version.tgz
@@ -61,7 +62,8 @@ tar -czvf espresso-$version.tar.gz espresso-$version/archive \
                                    espresso-$version/dev-tools \
                                    espresso-$version/Doc \
                                    espresso-$version/environment_variables \
-                                   espresso-$version/flib \
+                                   espresso-$version/LAXlib \
+                                   espresso-$version/FFTXlib \
                                    espresso-$version/Makefile \
                                    espresso-$version/include \
                                    espresso-$version/install \
@@ -77,7 +79,8 @@ tar -czvf espresso-$version.tar.gz espresso-$version/archive \
 
 cd espresso-$version
 tar -cvzf ../PWgui-$version.tar.gz    PWgui-$version
-tar -czvf ../PHonon-$version.tar.gz   PHonon PlotPhon QHA
+tar -czvf ../PHonon-$version.tar.gz   PHonon # PlotPhon QHA
+tar -czvf ../EPW-$version.tar.gz      EPW
 tar -czvf ../neb-$version.tar.gz      NEB
 tar -czvf ../pwcond-$version.tar.gz   PWCOND
 tar -czvf ../xspectra-$version.tar.gz XSpectra
@@ -86,3 +89,32 @@ tar -czvf ../GWW-$version.tar.gz      GWW
 tar -czvf ../tddfpt-$version.tar.gz   TDDFPT
 tar -czvf ../atomic-$version.tar.gz   atomic
 tar -czvf ../test-suite-$version.tar.gz test-suite
+
+
+# Generating and uploading documentation
+
+cd Doc
+
+for x in CPV PHonon NEB PP PW PWCOND TDDFPT atomic; \
+  do cp ../$x/Doc/*.xml .; cp ../$x/Doc/*.html .; cp ../$x/Doc/*.txt .; done
+
+cp ../PW/Doc/user_guide.pdf ./pw_user_guide.pdf
+cp ../CPV/Doc/user_guide.pdf ./cp_user_guide.pdf
+cp ../PP/Doc/user_guide.pdf ./pp_user_guide.pdf
+cp ../PHonon/Doc/user_guide.pdf ./ph_user_guide.pdf
+cp ../NEB/Doc/user_guide.pdf ./neb_user_guide.pdf
+cp ../atomic/Doc/pseudo-gen.pdf ./pseudo-gen.pdf
+
+cp -R ../PW/Doc/user_guide ./pw_user_guide
+cp -R ../CPV/Doc/user_guide ./cp_user_guide
+cp -R ../PP/Doc/user_guide ./pp_user_guide
+cp -R ../PHonon/Doc/user_guide ./ph_user_guide
+cp -R ../NEB/Doc/user_guide ./neb_user_guide
+cp -R ../atomic/Doc/pseudo-gen ./pseudo-gen
+
+#
+# Copy "Docs" to QE website
+
+scp -R Doc <...>@<...>/wp-content/uploads/Doc-$version
+
+# Connect to the website and create/update symbolic link to "Doc-$version"
