@@ -11,9 +11,10 @@ TOPDIR=`pwd`
 
 if test $# = 0
 then
-    dirs=" FFTXlib Modules clib PW/src CPV/src flib PW/tools upftools PP/src PWCOND/src\
+    dirs=" LAXlib FFTXlib Modules clib PW/src CPV/src PW/tools upftools PP/src \
+           PWCOND/src LR_Modules/ \
            PHonon/Gamma PHonon/PH PHonon/D3 PHonon/FD atomic/src XSpectra/src \
-           ACDFT NEB/src TDDFPT/src GIPAW/src GWW/pw4gww GWW/gww GWW/head" 
+           ACFDT/src NEB/src TDDFPT/src GIPAW/src GWW/pw4gww GWW/gww GWW/head GWW/bse" 
           
 elif
     test $1 = "-addson" 
@@ -46,39 +47,30 @@ for dir in $dirs; do
     # in directory DIR should be listed in DEPENDS
     LEVEL1=..
     LEVEL2=../..
-    DEPENDS="$LEVEL1/include $LEVEL1/iotk/src $LEVEL1/FFTXlib"
+    # default
+    DEPENDS="$LEVEL1/include" 
+    # for convenience, used later
+    DEPEND1="$LEVEL1/include $LEVEL1/iotk/src $LEVEL1/FFTXlib $LEVEL1/LAXlib"
+    DEPEND2="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/LAXlib $LEVEL2/Modules"
     case $DIR in 
-        flib | upftools )
-             DEPENDS="$LEVEL1/include $LEVEL1/iotk/src $LEVEL1/FFTXlib $LEVEL1/Modules" ;;
-	PP/src  )
-             DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules \
-                      $LEVEL2/PW/src" ;;
-	ACFDT ) 
-             DEPENDS="$LEVEL1/include $LEVEL1/iotk/src $LEVEL1/FFTXlib $LEVEL1/Modules \
-                      $LEVEL1/PW/src $LEVEL1/PHonon/PH" ;;
-	PW/src )
-	     DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules" ;;
-	PW/tools | PWCOND/src | PHonon/FD )
-	     DEPENDS="$LEVEL2/include $LEVEL2/PW/src $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules" ;;
-	CPV/src | atomic/src | GWW/gww )
-             DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules" ;;
-	PHonon/PH | PHonon/Gamma | XSpectra/src  | PWCOND/src | GWW/pw4gww | NEB/src | GIPAW/src )
-             DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules \
-                      $LEVEL2/PW/src" ;;
-	PHonon/D3 )
-	     DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules \
-	              $LEVEL2/PW/src $LEVEL2/PHonon/PH" ;;	
-        GWW/pw4gww )
-            DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules \
-                       $LEVEL2/PW/src  " ;;
-	GWW/gww )
-            DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules " ;;
-        GWW/head )
-             DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules \
-                      $LEVEL2/PW/src $LEVEL2/PHonon/PH " ;;
-	TDDFPT/src )
-             DEPENDS="$LEVEL2/include $LEVEL2/iotk/src $LEVEL2/FFTXlib $LEVEL2/Modules \
-                      $LEVEL2/PW/src $LEVEL2/PHonon/PH" ;;
+        Modules )
+             DEPENDS="$DEPEND1" ;;
+        upftools )
+             DEPENDS="$DEPEND1 $LEVEL1/Modules" ;;
+        LR_Modules )
+             DEPENDS="$DEPEND1 $LEVEL1/Modules $LEVEL1/PW/src" ;;
+	ACFDT/src ) 
+             DEPENDS="$DEPEND2 $LEVEL2/PW/src $LEVEL2/PHonon/PH $LEVEL2/LR_Modules" ;;
+	PW/src | CPV/src | atomic/src | GWW/gww )
+	     DEPENDS="$DEPEND2" ;;
+	PW/tools | PP/src | PWCOND/src | GWW/pw4gww | NEB/src )
+	     DEPENDS="$DEPEND2 $LEVEL2/PW/src" ;;
+	PHonon/FD | PHonon/PH | PHonon/Gamma | XSpectra/src  | GIPAW/src )
+	     DEPENDS="$DEPEND2 $LEVEL2/PW/src $LEVEL2/LR_Modules" ;;
+	PHonon/D3 | GWW/head | TDDFPT/src )
+	     DEPENDS="$DEPEND2 $LEVEL2/PW/src $LEVEL2/PHonon/PH $LEVEL2/LR_Modules" ;;	
+	GWW/bse )
+	 DEPENDS="$DEPEND2 $LEVEL2/PW/src $LEVEL2/PHonon/PH $LEVEL2/LR_Modulesi $LEVEL2/GWW/pw4gww $LEVEL2/GWW/gww" ;;	
     *)
 # if addson needs a make.depend file
 	DEPENDS="$DEPENDS $add_deps"
@@ -104,6 +96,12 @@ for dir in $dirs; do
             sed '/@mkl_dfti/d;/@fftw3.f/d;s/@fftw.c@/fftw.c/;s/@fft_param.f90@/fft_param.f90/' make.depend.tmp > make.depend
         fi
 
+        if test "$DIR" = "LAXlib"
+        then
+            sed '/@mpi@/d;/@elpa1@/d' make.depend > make.depend.tmp
+            sed 's/@la_param.f90@/la_param.f90/' make.depend.tmp > make.depend
+        fi
+
         if test "$DIR" = "Modules"
         then
             sed '/@mpi@/d;/@elpa1@/d' make.depend > make.depend.tmp
