@@ -24,9 +24,9 @@
 #include "iotk_auxmacros.h"
 
 
-#ifdef __IOTK_COMPLEX3
-#if 3 <= __IOTK_MAXRANK
-subroutine iotk_write_dat_COMPLEX3_3(unit,name,dat,dummy,attr,columns,sep,fmt,ierr)
+#ifdef __IOTK_COMPLEX4
+#if 6 <= __IOTK_MAXRANK
+subroutine iotk_write_dat_COMPLEX4_6(unit,name,dat,dummy,attr,columns,sep,fmt,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf, only : iotk_write_attr
@@ -36,10 +36,10 @@ subroutine iotk_write_dat_COMPLEX3_3(unit,name,dat,dummy,attr,columns,sep,fmt,ie
   use iotk_unit_interf
   use iotk_misc_interf
   implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
+  integer, parameter :: this_kind = iotk_COMPLEX4
   integer,                                    intent(in)  :: unit
   character(len=*),                           intent(in)  :: name
-  COMPLEX (kind=this_kind),           intent(in)  :: dat (:,:,:) 
+  COMPLEX (kind=this_kind),           intent(in)  :: dat (:,:,:,:,:,:) 
   type(iotk_dummytype),             optional              :: dummy
   character(len=*),                 optional, intent(in)  :: attr
   integer,                          optional, intent(in)  :: columns
@@ -259,7 +259,7 @@ call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
 !->
   allocate(dattmp(size(dat)))
 #if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dattmp,dat,size(dattmp),1)
+     call iotk_private_pack_COMPLEX4(dattmp,dat,size(dattmp),1)
 #else
      dattmp = pack(dat,mask=.true.)
 #endif
@@ -355,11 +355,11 @@ call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_write_dat_COMPLEX3_3
+end subroutine iotk_write_dat_COMPLEX4_6
 
 
 
-subroutine iotk_scan_dat_COMPLEX3_3(unit,name,dat,dummy,attr,found,default,ierr)
+subroutine iotk_scan_dat_COMPLEX4_6(unit,name,dat,dummy,attr,found,default,ierr)
   use iotk_base
 !-<
   use iotk_unit_interf
@@ -371,13 +371,13 @@ subroutine iotk_scan_dat_COMPLEX3_3(unit,name,dat,dummy,attr,found,default,ierr)
   use iotk_str_interf
   use iotk_misc_interf
   implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
+  integer, parameter :: this_kind = iotk_COMPLEX4
   integer,                                   intent(in)  :: unit
   character(len=*),                          intent(in)  :: name
 #ifdef __IOTK_WORKAROUND6
-  COMPLEX(kind=this_kind)                        :: dat (:,:,:)
+  COMPLEX(kind=this_kind)                        :: dat (:,:,:,:,:,:)
 #else
-  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:)
+  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:,:,:,:)
 #endif
   type(iotk_dummytype),            optional              :: dummy
 #ifdef __IOTK_WORKAROUND6
@@ -386,7 +386,7 @@ subroutine iotk_scan_dat_COMPLEX3_3(unit,name,dat,dummy,attr,found,default,ierr)
   character(len=*),                optional, intent(out) :: attr
 #endif
   logical,                         optional, intent(out) :: found
-  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:)
+  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:,:,:,:)
   integer,                         optional, intent(out) :: ierr
   COMPLEX (kind=this_kind),              allocatable :: tmpdat(:)
   integer :: ierrl,ierrl2
@@ -541,7 +541,7 @@ call iotk_error_write(ierrl,"rlen",rlen)
     goto 1
   end if
 #if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dat,tmpdat,size(tmpdat),1)
+     call iotk_private_pack_COMPLEX4(dat,tmpdat,size(tmpdat),1)
 #else
      dat = reshape(tmpdat,shape(dat))
 #endif
@@ -582,11 +582,11 @@ call iotk_error_write(ierrl,"name",name)
   else
     if(ierrl>0 .or. (.not.present(found) .and. .not.present(default))) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_scan_dat_COMPLEX3_3
+end subroutine iotk_scan_dat_COMPLEX4_6
 
 !-<
 ! ... new procedure: scan dat when you are already inside the data tag
-subroutine iotk_scan_dat_inside_COMPLEX3_3(unit,dat,dummy,found,default,ierr)
+subroutine iotk_scan_dat_inside_COMPLEX4_6(unit,dat,dummy,found,default,ierr)
   use iotk_base
   use iotk_unit_interf
   use iotk_attr_interf, only : iotk_scan_attr
@@ -596,17 +596,17 @@ subroutine iotk_scan_dat_inside_COMPLEX3_3(unit,dat,dummy,found,default,ierr)
   use iotk_str_interf
   use iotk_misc_interf
   implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
+  integer, parameter :: this_kind = iotk_COMPLEX4
   integer,                                   intent(in)  :: unit
 #ifdef __IOTK_WORKAROUND6
-  COMPLEX(kind=this_kind)                        :: dat (:,:,:)
+  COMPLEX(kind=this_kind)                        :: dat (:,:,:,:,:,:)
 #else
-  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:)
+  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:,:,:,:)
 #endif
 
   type(iotk_dummytype),            optional              :: dummy
   logical,                         optional, intent(out) :: found
-  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:)
+  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:,:,:,:)
   integer,                         optional, intent(out) :: ierr
   COMPLEX (kind=this_kind),              allocatable :: tmpdat(:)
   character(len=20) :: tmpstr
@@ -718,7 +718,7 @@ call iotk_error_write(ierrl,"rlen",rlen)
   end if
 
 #if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dat,tmpdat,size(tmpdat),1)
+     call iotk_private_pack_COMPLEX4(dat,tmpdat,size(tmpdat),1)
 #else
      dat = reshape(tmpdat,shape(dat))
 #endif
@@ -750,15 +750,15 @@ call iotk_error_write(ierrl,"rname",rname)
     if(ierrl>0 .or. (.not.present(found) .and. .not.present(default))) call iotk_error_handler(ierrl)
   end if
 
-end subroutine iotk_scan_dat_inside_COMPLEX3_3
+end subroutine iotk_scan_dat_inside_COMPLEX4_6
 !->
 
 #endif
 #endif
 
-subroutine iotk_dat_dummy_COMPLEX3_3
+subroutine iotk_dat_dummy_COMPLEX4_6
   write(0,*)
-end subroutine iotk_dat_dummy_COMPLEX3_3
+end subroutine iotk_dat_dummy_COMPLEX4_6
 
 
 
@@ -772,9 +772,9 @@ end subroutine iotk_dat_dummy_COMPLEX3_3
 #include "iotk_auxmacros.h"
 
 
-#ifdef __IOTK_COMPLEX3
-#if 4 <= __IOTK_MAXRANK
-subroutine iotk_write_dat_COMPLEX3_4(unit,name,dat,dummy,attr,columns,sep,fmt,ierr)
+#ifdef __IOTK_COMPLEX4
+#if 7 <= __IOTK_MAXRANK
+subroutine iotk_write_dat_COMPLEX4_7(unit,name,dat,dummy,attr,columns,sep,fmt,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf, only : iotk_write_attr
@@ -784,10 +784,10 @@ subroutine iotk_write_dat_COMPLEX3_4(unit,name,dat,dummy,attr,columns,sep,fmt,ie
   use iotk_unit_interf
   use iotk_misc_interf
   implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
+  integer, parameter :: this_kind = iotk_COMPLEX4
   integer,                                    intent(in)  :: unit
   character(len=*),                           intent(in)  :: name
-  COMPLEX (kind=this_kind),           intent(in)  :: dat (:,:,:,:) 
+  COMPLEX (kind=this_kind),           intent(in)  :: dat (:,:,:,:,:,:,:) 
   type(iotk_dummytype),             optional              :: dummy
   character(len=*),                 optional, intent(in)  :: attr
   integer,                          optional, intent(in)  :: columns
@@ -1007,7 +1007,7 @@ call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
 !->
   allocate(dattmp(size(dat)))
 #if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dattmp,dat,size(dattmp),1)
+     call iotk_private_pack_COMPLEX4(dattmp,dat,size(dattmp),1)
 #else
      dattmp = pack(dat,mask=.true.)
 #endif
@@ -1103,11 +1103,11 @@ call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_write_dat_COMPLEX3_4
+end subroutine iotk_write_dat_COMPLEX4_7
 
 
 
-subroutine iotk_scan_dat_COMPLEX3_4(unit,name,dat,dummy,attr,found,default,ierr)
+subroutine iotk_scan_dat_COMPLEX4_7(unit,name,dat,dummy,attr,found,default,ierr)
   use iotk_base
 !-<
   use iotk_unit_interf
@@ -1119,13 +1119,13 @@ subroutine iotk_scan_dat_COMPLEX3_4(unit,name,dat,dummy,attr,found,default,ierr)
   use iotk_str_interf
   use iotk_misc_interf
   implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
+  integer, parameter :: this_kind = iotk_COMPLEX4
   integer,                                   intent(in)  :: unit
   character(len=*),                          intent(in)  :: name
 #ifdef __IOTK_WORKAROUND6
-  COMPLEX(kind=this_kind)                        :: dat (:,:,:,:)
+  COMPLEX(kind=this_kind)                        :: dat (:,:,:,:,:,:,:)
 #else
-  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:,:)
+  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:,:,:,:,:)
 #endif
   type(iotk_dummytype),            optional              :: dummy
 #ifdef __IOTK_WORKAROUND6
@@ -1134,7 +1134,7 @@ subroutine iotk_scan_dat_COMPLEX3_4(unit,name,dat,dummy,attr,found,default,ierr)
   character(len=*),                optional, intent(out) :: attr
 #endif
   logical,                         optional, intent(out) :: found
-  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:,:)
+  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:,:,:,:,:)
   integer,                         optional, intent(out) :: ierr
   COMPLEX (kind=this_kind),              allocatable :: tmpdat(:)
   integer :: ierrl,ierrl2
@@ -1289,7 +1289,7 @@ call iotk_error_write(ierrl,"rlen",rlen)
     goto 1
   end if
 #if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dat,tmpdat,size(tmpdat),1)
+     call iotk_private_pack_COMPLEX4(dat,tmpdat,size(tmpdat),1)
 #else
      dat = reshape(tmpdat,shape(dat))
 #endif
@@ -1330,11 +1330,11 @@ call iotk_error_write(ierrl,"name",name)
   else
     if(ierrl>0 .or. (.not.present(found) .and. .not.present(default))) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_scan_dat_COMPLEX3_4
+end subroutine iotk_scan_dat_COMPLEX4_7
 
 !-<
 ! ... new procedure: scan dat when you are already inside the data tag
-subroutine iotk_scan_dat_inside_COMPLEX3_4(unit,dat,dummy,found,default,ierr)
+subroutine iotk_scan_dat_inside_COMPLEX4_7(unit,dat,dummy,found,default,ierr)
   use iotk_base
   use iotk_unit_interf
   use iotk_attr_interf, only : iotk_scan_attr
@@ -1344,17 +1344,17 @@ subroutine iotk_scan_dat_inside_COMPLEX3_4(unit,dat,dummy,found,default,ierr)
   use iotk_str_interf
   use iotk_misc_interf
   implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
+  integer, parameter :: this_kind = iotk_COMPLEX4
   integer,                                   intent(in)  :: unit
 #ifdef __IOTK_WORKAROUND6
-  COMPLEX(kind=this_kind)                        :: dat (:,:,:,:)
+  COMPLEX(kind=this_kind)                        :: dat (:,:,:,:,:,:,:)
 #else
-  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:,:)
+  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:,:,:,:,:)
 #endif
 
   type(iotk_dummytype),            optional              :: dummy
   logical,                         optional, intent(out) :: found
-  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:,:)
+  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:,:,:,:,:)
   integer,                         optional, intent(out) :: ierr
   COMPLEX (kind=this_kind),              allocatable :: tmpdat(:)
   character(len=20) :: tmpstr
@@ -1466,7 +1466,7 @@ call iotk_error_write(ierrl,"rlen",rlen)
   end if
 
 #if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dat,tmpdat,size(tmpdat),1)
+     call iotk_private_pack_COMPLEX4(dat,tmpdat,size(tmpdat),1)
 #else
      dat = reshape(tmpdat,shape(dat))
 #endif
@@ -1498,763 +1498,15 @@ call iotk_error_write(ierrl,"rname",rname)
     if(ierrl>0 .or. (.not.present(found) .and. .not.present(default))) call iotk_error_handler(ierrl)
   end if
 
-end subroutine iotk_scan_dat_inside_COMPLEX3_4
+end subroutine iotk_scan_dat_inside_COMPLEX4_7
 !->
 
 #endif
 #endif
 
-subroutine iotk_dat_dummy_COMPLEX3_4
+subroutine iotk_dat_dummy_COMPLEX4_7
   write(0,*)
-end subroutine iotk_dat_dummy_COMPLEX3_4
-
-
-
-
-
-!------------------------------------------------------------------------------!
-! Inclusion of configuration file
-#include "iotk_config.h"
-!------------------------------------------------------------------------------!
-
-#include "iotk_auxmacros.h"
-
-
-#ifdef __IOTK_COMPLEX3
-#if 5 <= __IOTK_MAXRANK
-subroutine iotk_write_dat_COMPLEX3_5(unit,name,dat,dummy,attr,columns,sep,fmt,ierr)
-  use iotk_base
-  use iotk_error_interf
-  use iotk_attr_interf, only : iotk_write_attr
-  use iotk_write_interf
-  use iotk_fmt_interf
-  use iotk_str_interf
-  use iotk_unit_interf
-  use iotk_misc_interf
-  implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
-  integer,                                    intent(in)  :: unit
-  character(len=*),                           intent(in)  :: name
-  COMPLEX (kind=this_kind),           intent(in)  :: dat (:,:,:,:,:) 
-  type(iotk_dummytype),             optional              :: dummy
-  character(len=*),                 optional, intent(in)  :: attr
-  integer,                          optional, intent(in)  :: columns
-  character(len=*),                 optional, intent(in)  :: sep
-  character(len=*),                 optional, intent(in)  :: fmt
-  integer,                          optional, intent(out) :: ierr
-  integer :: ierrl,lunit,iostat
-  logical :: binary,raw,stream
-  integer :: lcolumns
-!-<
-  integer :: dat_rank,i
-  integer, dimension(:), allocatable :: dat_shape
-  logical :: qe_syntax
-!->
-  integer(iotk_header_kind), parameter :: idummy=0
-  character(100) :: lsep
-  character(300) :: usefmt
-  character(iotk_attlenx) :: lattr
-  character(iotk_attlenx) :: attr_tmp
-!-<
-  character(len=10) :: tmpstr
-!->
-  type (iotk_unit), pointer :: this
-  COMPLEX (kind=this_kind),allocatable :: dattmp(:)
-  integer :: itmp
-  ierrl = 0
-  iostat = 0
-  lcolumns = 1
-  lsep(1:2) = " "//iotk_eos
-  if(present(columns)) lcolumns = columns
-  if(present(sep)) then
-    call iotk_strcpy(lsep,sep,ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-  end if
-  lunit = iotk_phys_unit(unit)
-  call iotk_unit_get(lunit,pointer=this)
-!-<  
-  qe_syntax = .false.
-  if (associated(this)) then
-     qe_syntax = this%qe_syntax
-  end if
-!->
-  raw = .false.
-  if(associated(this)) then
-    raw = this%raw
-  end if
-  call iotk_inquire(lunit,binary,stream,ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  call iotk_strcpy(usefmt,"!",ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  if(present(fmt) .and. .not. raw) call iotk_strcpy(usefmt,iotk_strtrim(fmt),ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  if(iotk_strscan(usefmt,"<>&")/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-call iotk_error_msg(ierrl,'Special characters (<>&) found in fmt string')
-call iotk_error_write(ierrl,"unit",unit)
-call iotk_error_write(ierrl,"name",trim(name))
-call iotk_error_write(ierrl,"fmt",trim(fmt))
-    goto 1
-  end if
-!-<
-  if (.not.qe_syntax) then
-!->
-  call iotk_write_attr(lattr,"type",iotk_tolower("COMPLEX"),first=.true.,ierr=ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  call iotk_write_attr(lattr,"size",size(dat),ierr=ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-!-<
-  end if
-!->
-!-<
-  if (qe_syntax) then
-    lattr(1:1) = iotk_eos
-    dat_rank = size(shape(dat))
-    if (dat_rank>0) then
-       allocate(dat_shape(dat_rank))
-       dat_shape = shape(dat)
-       call iotk_write_attr(lattr,"rank",dat_rank,ierr=ierrl,first=.true.)
-       if(ierrl/=0) then
-          call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-          goto 1
-       end if
-       do i=1,dat_rank
-       write(tmpstr,'(i10)') i
-  tmpstr = adjustl(tmpstr)
-          call iotk_write_attr(lattr,"n"//tmpstr(1:len_trim(tmpstr)),dat_shape(i),ierr=ierrl)
-          if(ierrl/=0) then
-             call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-             goto 1
-          end if
-       end do
-       deallocate(dat_shape)
-    end if
-  end if
-!->
-  if(binary) then
-    call iotk_write_attr(lattr,"kind",kind(dat),ierr=ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-  end if
-  if(.not.iotk_strcomp(usefmt,"!") .and. .not.qe_syntax) call iotk_write_attr(lattr,"fmt",iotk_strtrim(usefmt),ierr=ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  if(lcolumns/=1) call iotk_write_attr(lattr,"columns",lcolumns,ierr=ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-!-<
-  if (.not.qe_syntax) then
-!->
-  if(present(attr)) then
-    attr_tmp(1:1)=iotk_eos
-    call iotk_strcpy(attr_tmp,attr,ierr=ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-    call iotk_delete_attr(attr_tmp,"type",ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-    call iotk_delete_attr(attr_tmp,"kind",ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-    call iotk_delete_attr(attr_tmp,"size",ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-    call iotk_delete_attr(attr_tmp,"fmt",ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-    call iotk_delete_attr(attr_tmp,"columns",ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-    call iotk_delete_attr(attr_tmp,"len",ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-    if(iotk_strlen_trim(attr_tmp)>0) call iotk_strcat(lattr,iotk_strtrim(attr_tmp),ierr=ierrl)
-  end if
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  call iotk_write_begin(unit,name,lattr,ierr=ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-!-<
-  else
-    if(present(attr)) then
-      call iotk_write_begin(unit,name,attr,ierr=ierrl)
-    else
-      call iotk_write_begin(unit,name,ierr=ierrl)
-    end if
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-    call iotk_write_begin(unit,iotk_tolower("COMPLEX"),lattr,ierr=ierrl,new_line=.true.)
-  end if
-!->
-  allocate(dattmp(size(dat)))
-#if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dattmp,dat,size(dattmp),1)
-#else
-     dattmp = pack(dat,mask=.true.)
-#endif
-  if(binary) then
-    if(raw) then
-      write(lunit,iostat=iostat) (dattmp(itmp),itmp=1,size(dattmp))
-      if(iostat/=0) then
-        call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        goto 1
-      end if
-    else
-      write(lunit,iostat=iostat) idummy,(dattmp(itmp),itmp=1,size(dattmp))
-      if(iostat/=0) then
-        call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        goto 1
-      end if
-    end if
-  else
-    if(raw) then
-      write(lunit,*,iostat=iostat) (dattmp(itmp),itmp=1,size(dattmp))
-      if(iostat/=0) then
-        call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        goto 1
-      end if
-    else if(iotk_strcomp(usefmt,"*")) then
-      write(lunit,*,iostat=iostat) (dattmp(itmp),itmp=1,size(dattmp))
-      if(iostat/=0) then
-        call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        goto 1
-      end if
-    else if(iotk_strcomp(usefmt,"!")) then
-!-<
-    if ((.not. qe_syntax) .or. (size(dattmp)>1)) then
-     write(lunit,fmt=trim(iotk_wfmt("COMPLEX",kind(dattmp),lcolumns,-1,lsep)),iostat=iostat) (dattmp(itmp),itmp=1,size(dattmp))
-     if(iostat/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-     end if
-    else
-     write(lunit,fmt=trim(iotk_wfmt("COMPLEX",kind(dattmp),lcolumns,-1,lsep)),advance='no',iostat=iostat) dattmp(1)
-     if(iostat/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-     end if
-    end if
-!->
-    else
-!-<
-    if ((.not. qe_syntax) .or. (size(dattmp)>1)) then
-      write(lunit,fmt=usefmt(1:iotk_strlen(usefmt)),iostat=iostat) (dattmp(itmp),itmp=1,size(dattmp))
-      if(iostat/=0) then
-        call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        goto 1
-      end if
-    else
-      write(lunit,fmt=usefmt(1:iotk_strlen(usefmt)),advance='no',iostat=iostat) dattmp(1)
-      if(iostat/=0) then
-        call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        goto 1
-      end if
-    end if
-!->
-    end if
-  end if
-!-<
-  if (qe_syntax) then
-    call iotk_write_end(unit,iotk_tolower("COMPLEX"),indentation=.true.,ierr=ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-      goto 1
-    end if
-  end if
-!->
-  call iotk_write_end(unit,name,ierr=ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-1 continue
-  if(allocated(dattmp)) deallocate(dattmp)
-  if(present(ierr)) then
-    ierr = ierrl
-  else
-    if(ierrl/=0) call iotk_error_handler(ierrl)
-  end if
-end subroutine iotk_write_dat_COMPLEX3_5
-
-
-
-subroutine iotk_scan_dat_COMPLEX3_5(unit,name,dat,dummy,attr,found,default,ierr)
-  use iotk_base
-!-<
-  use iotk_unit_interf
-  use iotk_attr_interf, only : iotk_scan_attr
-!->
-  use iotk_error_interf
-  use iotk_dat_interf, only: iotk_scan_dat_aux
-  use iotk_scan_interf
-  use iotk_str_interf
-  use iotk_misc_interf
-  implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
-  integer,                                   intent(in)  :: unit
-  character(len=*),                          intent(in)  :: name
-#ifdef __IOTK_WORKAROUND6
-  COMPLEX(kind=this_kind)                        :: dat (:,:,:,:,:)
-#else
-  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:,:,:)
-#endif
-  type(iotk_dummytype),            optional              :: dummy
-#ifdef __IOTK_WORKAROUND6
-  character(len=*),                optional              :: attr
-#else
-  character(len=*),                optional, intent(out) :: attr
-#endif
-  logical,                         optional, intent(out) :: found
-  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:,:,:)
-  integer,                         optional, intent(out) :: ierr
-  COMPLEX (kind=this_kind),              allocatable :: tmpdat(:)
-  integer :: ierrl,ierrl2
-  integer :: rkind,rsize,rlen
-  character(iotk_vallenx) :: rtype
-  character(iotk_vallenx) :: fmt
-  character(iotk_attlenx) :: lattr
-!-<
-! ... necessary because i need the syntax to use
-  type (iotk_unit), pointer :: this
-! ... necessary to read the tag describing the type
-  character(iotk_taglenx) :: ltag
-  character(iotk_namlenx) :: r_name
-  character(iotk_attlenx) :: lattr2
-  character(len=20) :: tmpstr
-! ... necessary for scan_tag
-  logical :: binary,stream,qe_syntax
-  integer :: r_control,rrank
-  integer :: rshape,i
-!->
-  integer :: columns
-  logical :: inside,foundl
-!-<
-  call iotk_unit_get(iotk_phys_unit(unit),pointer=this)
-  qe_syntax = .false.
-  if (associated(this)) THEN
-     qe_syntax = this%qe_syntax
-  end if    
-!->
-  inside = .false.
-  ierrl = 0
-  ierrl2 = 0
-  foundl=.false.
-  call iotk_scan_begin(unit,name,lattr,found=foundl,ierr=ierrl)
-  if(.not. foundl) goto 1
-  foundl = .true.
-  inside = .true.
-  if(present(attr)) call iotk_strcpy(attr,lattr,ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-!-<
-  if (.not.qe_syntax) then
-!-<
-  call iotk_parse_dat(lattr,rtype,rkind,rsize,rlen,fmt,columns,ierrl)
-! Note that columns is not effectively used
-  if(ierrl/=0) goto 1
-!-<
-  else
-     call iotk_inquire(iotk_phys_unit(unit),binary,stream,ierrl)
-     if(ierrl/=0) then
-        call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        goto 1
-     end if
-     do
-        call iotk_scan_tag(unit,+1,r_control,ltag,binary,stream,ierrl)
-        if(ierrl/=0) then
-           call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-           goto 1
-        end if
-       if (r_control==4) then
-   cycle
-else
-   exit
-end if
-     end do
-
-     call iotk_tag_parse(ltag,r_name,lattr2,ierrl)
-     if(ierrl/=0) then
-        call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-goto 1
-     end if
-     call iotk_strcpy(rtype,r_name,ierrl)
-     rtype = iotk_toupper(rtype)
-
-      rlen = -1
-
-     if (rtype(1:iotk_strlen(rtype))/="STRING") then
-        call iotk_scan_attr(lattr2,"rank",rrank,ierr=ierrl,default=0)
-        if(ierrl/=0) then
-           call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-           return
-        end if
-        rsize = 1
-        if (rrank>0) then
-           do i=1,rrank
-           write(tmpstr,'(i20)') i
-      tmpstr = adjustl(tmpstr)
-      call iotk_scan_attr(lattr2,"n"//tmpstr(1:len_trim(tmpstr)),rshape,ierr=ierrl,default=0)
-              if(ierrl/=0) then
-                 call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-                 return
-              end if
-      rsize = rsize*rshape
-   end do
-end if
-     else
-        call iotk_strcpy(rtype,"CHARACTER",ierrl)
-        rsize = -1        
-     end if
-
-     call iotk_scan_attr(lattr2,"kind",rkind,ierr=ierrl,default=-1)
-     if(ierrl/=0) then
-        call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        return
-     end if
-     call iotk_scan_attr(lattr2,"fmt", fmt, ierr=ierrl,eos=.true.,default="!"//iotk_eos)
-     if(ierrl/=0) then
-        call iotk_error_issue(ierr,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.27 ")
-        return
-     end if
-     call iotk_scan_attr(lattr2,"columns",columns,ierr=ierrl,default=1)
-     if(ierrl/=0) then
-       call iotk_error_issue(ierr,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.27 ")
-       return
-     end if
-  end if
-!->
-  if(.not. (iotk_strcomp(rtype,iotk_eos) .or. iotk_strcomp(rtype,"COMPLEX") ) ) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-call iotk_error_msg(ierrl,' ')
-call iotk_error_write(ierrl,"rtype",rtype(1:iotk_strlen(rtype)))
-call iotk_error_write(ierrl,"type","COMPLEX")
-    goto 1
-  end if
-  if(.not. (rsize==-1 .or. rsize==size(dat)) ) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  if(rkind==-1) rkind = kind(dat)
-  allocate(tmpdat(size(dat)))
-  call iotk_scan_dat_aux(unit,tmpdat,rkind,rlen,fmt(1:iotk_strlen(fmt)),ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-call iotk_error_msg(ierrl,'Error reading data')
-call iotk_error_write(ierrl,"name",name)
-call iotk_error_write(ierrl,"rkind",rkind)
-call iotk_error_write(ierrl,"rlen",rlen)
-    goto 1
-  end if
-#if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dat,tmpdat,size(tmpdat),1)
-#else
-     dat = reshape(tmpdat,shape(dat))
-#endif
-1 continue
-!-<
-  if ( allocated(tmpdat) ) deallocate(tmpdat)
-!->
-  if(inside) then
-!-<
-    if (qe_syntax) then 
-      call iotk_scan_end(unit,iotk_tolower("COMPLEX"),ierr=ierrl2) 
-       if(ierrl2/=0) then
-          call iotk_error_clear(ierrl)
-          ierrl=ierrl2
-       end if
-    end if
-!->
-    call iotk_scan_end(unit,name,ierr=ierrl2)
-    if(ierrl2/=0) then
-      call iotk_error_clear(ierrl)
-      ierrl=ierrl2
-    end if
-  end if
-  if(ierrl/=0) foundl=.false.
-  if(present(found)) found = foundl
-  if(ierrl==0 .and. .not. present(found) .and. .not. present(default) .and. .not. foundl) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-call iotk_error_msg(ierrl,'Dat not found')
-call iotk_error_write(ierrl,"name",name)
-    ierrl = - ierrl
-  end if 
-  if(present(default) .and. .not. foundl) then
-    dat=default
-  end if
-  if(present(ierr)) then
-    ierr = ierrl
-  else
-    if(ierrl>0 .or. (.not.present(found) .and. .not.present(default))) call iotk_error_handler(ierrl)
-  end if
-end subroutine iotk_scan_dat_COMPLEX3_5
-
-!-<
-! ... new procedure: scan dat when you are already inside the data tag
-subroutine iotk_scan_dat_inside_COMPLEX3_5(unit,dat,dummy,found,default,ierr)
-  use iotk_base
-  use iotk_unit_interf
-  use iotk_attr_interf, only : iotk_scan_attr
-  use iotk_error_interf
-  use iotk_dat_interf, only: iotk_scan_dat_aux
-  use iotk_scan_interf
-  use iotk_str_interf
-  use iotk_misc_interf
-  implicit none
-  integer, parameter :: this_kind = iotk_COMPLEX3
-  integer,                                   intent(in)  :: unit
-#ifdef __IOTK_WORKAROUND6
-  COMPLEX(kind=this_kind)                        :: dat (:,:,:,:,:)
-#else
-  COMPLEX(kind=this_kind),           intent(out) :: dat (:,:,:,:,:)
-#endif
-
-  type(iotk_dummytype),            optional              :: dummy
-  logical,                         optional, intent(out) :: found
-  COMPLEX(kind=this_kind), optional, intent(in)  :: default (:,:,:,:,:)
-  integer,                         optional, intent(out) :: ierr
-  COMPLEX (kind=this_kind),              allocatable :: tmpdat(:)
-  character(len=20) :: tmpstr
-  integer :: rrank
-  integer :: rshape,i
-  integer :: ierrl,ierrl2
-  integer :: rkind,rsize,rlen
-  character(iotk_vallenx) :: rtype
-  character(iotk_vallenx) :: fmt
-  character(iotk_attlenx) :: lattr
-  character(iotk_namlenx) :: rname
-  type (iotk_unit), pointer :: this
-  integer :: columns
-  logical :: inside,foundl,qe_syntax
-
-
-  call iotk_unit_get(iotk_phys_unit(unit),pointer=this)
-
-  qe_syntax = .true.
-  IF (associated(this)) then
-     qe_syntax = this%qe_syntax
-  END IF
-
-  inside = .false.
-  ierrl = 0
-  ierrl2 = 0
-  foundl=.false.
-
-  if (.not.qe_syntax) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-
-  rname = iotk_tolower("COMPLEX")
-  call iotk_scan_begin(unit,iotk_tolower("COMPLEX"),lattr,found=foundl,ierr=ierrl)
-  if(.not. foundl) goto 1
-
-
-  foundl = .true.
-  inside = .true.
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  
-  rlen = -1
-
-
-     call iotk_scan_attr(lattr,"rank",rrank,ierr=ierrl,default=0)
-     if(ierrl/=0) then
-        call iotk_error_issue(ierrl,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-        return
-     end if
-     rsize = 1
-     if (rrank>0) then
-        do i=1,rrank
-   write(tmpstr,'(i20)') i
-   tmpstr = adjustl(tmpstr)
-           call iotk_scan_attr(lattr,"n"//tmpstr(1:len_trim(tmpstr)),rshape,ierr=ierrl,default=0)
-           if(ierrl/=0) then
-              call iotk_error_issue(ierrl,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-              return
-           end if
-           rsize = rsize*rshape
-        end do
-     end if
-
-
-  call iotk_scan_attr(lattr,"kind",rkind,ierr=ierrl,default=-1)
-  if(ierrl/=0) then
-     call iotk_error_issue(ierrl,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-     return
-  end if
-  call iotk_scan_attr(lattr,"fmt", fmt, ierr=ierrl,eos=.true.,default="!"//iotk_eos)
-  if(ierrl/=0) then
-     call iotk_error_issue(ierr,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.27 ")
-     return
-  end if
-  call iotk_scan_attr(lattr,"columns",columns,ierr=ierrl,default=1)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierr,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.27 ")
-    return
-  end if
-
-  if(.not. (rsize==-1 .or. rsize==size(dat)) ) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-    goto 1
-  end if
-  if(rkind==-1) rkind = kind(dat)
-
-  allocate(tmpdat(size(dat)))
-  call iotk_scan_dat_aux(unit,tmpdat,rkind,rlen,fmt(1:iotk_strlen(fmt)),ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-call iotk_error_msg(ierrl,'Error reading data')
-call iotk_error_write(ierrl,"rname",rname)
-call iotk_error_write(ierrl,"rkind",rkind)
-call iotk_error_write(ierrl,"rlen",rlen)
-    goto 1
-  end if
-
-#if defined(__IOTK_WORKAROUND3) || defined(__IOTK_WORKAROUND4)
-     call iotk_private_pack_COMPLEX3(dat,tmpdat,size(tmpdat),1)
-#else
-     dat = reshape(tmpdat,shape(dat))
-#endif
-  deallocate(tmpdat)
-1 continue
-  if(inside) then
-     call iotk_scan_end(unit,iotk_tolower("COMPLEX"),ierr=ierrl2) 
-     if(ierrl2/=0) then
-        call iotk_error_clear(ierrl)
-        ierrl=ierrl2
-     end if
-  end if
-
-  if(ierrl/=0) foundl=.false.
-  if(present(found)) found = foundl
-  if(ierrl==0 .and. .not. present(found) .and. .not. present(default) .and. .not. foundl) then
-    call iotk_error_issue(ierrl,"iotk_scan_dat_inside",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.27 ")
-call iotk_error_msg(ierrl,'Dat not found')
-call iotk_error_write(ierrl,"rname",rname)
-    ierrl = - ierrl
-  end if 
-  if(present(default) .and. .not. foundl) then
-    dat=default
-  end if
-  if(present(ierr)) then
-    ierr = ierrl
-  else
-    if(ierrl>0 .or. (.not.present(found) .and. .not.present(default))) call iotk_error_handler(ierrl)
-  end if
-
-end subroutine iotk_scan_dat_inside_COMPLEX3_5
-!->
-
-#endif
-#endif
-
-subroutine iotk_dat_dummy_COMPLEX3_5
-  write(0,*)
-end subroutine iotk_dat_dummy_COMPLEX3_5
+end subroutine iotk_dat_dummy_COMPLEX4_7
 
 
 
